#!/usr/bin/env python3
################################################################################
#
# get_SN_cube.py
# ----------------------
# Perl script: get_SN_cube.pl
#
################################################################################

# System imports
import sys
from os.path import basename
from copy import deepcopy as copy

### Local imports
from pyFIT3D.common.io import ReadArguments
from pyFIT3D.common.tools import img2spec_e

class ReadArgumentsLocal(ReadArguments):
    """
    Local argument parser
    """
    __script_name__ = basename(sys.argv[0])
    __mandatory__ = ['input_fits', 'ny', 'output']
    __optional__ = []
    __arg_names__ = __mandatory__ + __optional__
    __N_tot_args__ = len(__arg_names__)
    __conv_func_mandatory__ = {'input_fits': str, 'ny': int, 'output': str}
    __conv_func_optional__ = {}
    __conv_func__ = __conv_func_mandatory__.copy()
    __conv_func__.update(__conv_func_optional__)

    __usage_msg__ = f'USE: {__script_name__}'
    __usage_msg__ += ' INPUT_FILE.fits NY OUTPUT_FILE.txt'

    def __init__(self, args_list=None, verbose=False):
        ReadArguments.__init__(self, args_list, verbose=verbose)

if __name__ == '__main__':
    parsed_args = ReadArgumentsLocal()
    img2spec_e(parsed_args.input_fits, parsed_args.ny, parsed_args.output)



#!/usr/bin/env python3
import os
import sys
import errno
import itertools
import numpy as np
import pandas as pd
from os.path import basename, isfile, dirname, join

import matplotlib as mpl
mpl.rcParams['text.usetex'] = True
mpl.rcParams['font.family'] = 'serif'
mpl.rcParams['font.serif'] = 'Times New Roman'
mpl.rcParams['axes.unicode_minus'] = False
mpl.rcParams['legend.numpoints'] = 1
from matplotlib import pyplot as plt
import seaborn as sns
sns.set(context='paper', style='ticks', palette='colorblind', color_codes=True,
        font_scale=1.5)

from pyFIT3D.common.auto_ssp_tools import ConfigAutoSSP
from pyFIT3D.common.gas_tools import read_fit_elines_output
from pyFIT3D.common.io import trim_waves, sel_waves, print_verbose
from pyFIT3D.common.io import ReadArguments, readFileArgumentParser
from pyFIT3D.common.io import get_data_from_fits, get_wave_from_header
from pyFIT3D.common.constants import __c__, __mask_elines_window__
from pyFIT3D.common.constants import _MODELS_ELINE_PAR, __sigma_to_FWHM__

__solar_metallicity__ = 0.019

latex_ppi = 72.0

latex_column_width_pt = 240.0
latex_column_width = latex_column_width_pt/latex_ppi

latex_text_width_pt = 504.0
latex_text_width = latex_text_width_pt/latex_ppi

golden_mean = 0.5 * (1. + 5**0.5)
aspect = 1/golden_mean

def parse_arguments(default_args_file=None):
    default_args = {
        'mask_list': None,
        'elines_mask_file': None,
        'img_suffix': 'pdf',
        'dpi': 300,
        'cmap': 'Blues',
        'wl_range': [3800, 6850],
        'fraction': False,
        'instrumental_dispersion': None,
        'block': False,
    }

    help_args = {
        'fraction': 'Display coeffs in fractions instead percent.',
        'block': 'Block image at the end waiting for the user to close the window.',
    }

    parser = readFileArgumentParser(fromfile_prefix_chars='@')
    parser.add_argument('--mask_list', metavar='FILE', type=str,
                        default=default_args['mask_list'])
    parser.add_argument('--elines_mask_file', metavar='FILE', type=str,
                        default=default_args['elines_mask_file'])
    parser.add_argument('--img_suffix', '-i', metavar='IMG_SUFFIX', type=str,
                        default=default_args['img_suffix'])
    parser.add_argument('--dpi', metavar='INT', type=int, default=default_args['dpi'])
    parser.add_argument('--cmap', metavar='COLORMAP', type=str,
                        default=default_args['cmap'])
    parser.add_argument('--wl_range', nargs=2, metavar='INT', type=int)
    parser.add_argument('--fraction', action='store_true',
                        default=default_args['fraction'], help=help_args['fraction'])
    parser.add_argument('--block', action='store_true', default=default_args['block'],
                        help=help_args['block'])
    parser.add_argument('--instrumental_dispersion', '-d', metavar='FLOAT', type=float,
                        default=default_args['instrumental_dispersion'])
    # positional arguments
    parser.add_argument('out_file', metavar='OUT_FILE', type=str)
    parser.add_argument('--verbose', '-v', action='count', default=0)

    args_list = sys.argv[1:]
    # if exists file default.args, load default args
    if default_args_file is not None and isfile(default_args_file):
        args_list.insert(0, '@%s' % default_args_file)
    args = parser.parse_args(args=args_list)
    # TREAT ARGUMENTS HERE
    # parse filenames
    dname = os.path.dirname(args.out_file)
    if dname:
        _bn = basename(args.out_file)
    else:
        _bn =args.out_file

    args.out_file_fit = join(dname, 'output.' + _bn + '.fits')
    if not isfile(args.out_file_fit):
        args.out_file_fit += '.gz'
    args.out_file_elines = join(dname, 'elines_' + _bn)
    args.out_file_elines = 'elines_' + args.out_file
    args.out_file_coeffs = join(dname, 'coeffs_' + _bn)
    for _f in [args.out_file, args.out_file_fit, args.out_file_coeffs]:
        if not isfile(_f):
            raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), _f)
    return args

def plot_output_resume(wl, spec__tw, output_results, coeffs, masks, e_masks,
                       el_wl_fit, wl_range=[3800, 6850], cmap='Blues',
                       vmin=0.1, vmax=50, sigma_unit='km/s', percent=True):
    fmt = '.1f' if percent else '.3f'
    wlmsk = np.ones_like(wl, dtype='bool')
    if wl_range is not None:
        wlmsk = trim_waves(wl, wl_range)
    else:
        wl_range = [wl.min(), wl.max()]

    f = plt.figure()
    width = 2*latex_text_width
    f.set_size_inches(width, 0.7*width*aspect)

    gs = f.add_gridspec(3,4)
    ax_sp = f.add_subplot(gs[:2, :3])
    ax = f.add_subplot(gs[2, :])

    ax_lw = f.add_subplot(gs[:2, 3])
    ax_lw.set_axis_off()

    msg = r'$\chi^2$ = %6.2f' % output_results[0]
    msg += '\n'
    msg += r'RMS Flux = %6.2f' % output_results[13]
    msg += '\n'
    msg += '\n'
    msg += r'sys vel = %6.2f +/- %.2f km/s' % (output_results[7]*__c__, output_results[8]*__c__)
    msg += '\n'
    msg += r'$\sigma^\star$ = %6.2f +/- %0.2f' % (output_results[9], output_results[10])
    msg += sigma_unit
    msg += '\n'
    msg += r'${\rm A}_{\rm V}^\star$ = %6.2f +/- %0.2f' % (output_results[5], output_results[6])
    msg += '\n'
    msg += '\n'
    msg += r'$\left< \log {\rm t^\star} \right>_{\rm L}$ = %6.2f +/- %0.2f Gyr' % (output_results[1], output_results[2])
    msg += '\n'
    msg += r'$\left< \log {\rm Z}^\star \right>_{\rm L}$ = %6.2f +/- %0.2f [Z/H]' % (output_results[3], output_results[4])
    msg += '\n'
    msg += r'$\left< \log {\rm t^\star} \right>_{\rm M}$ = %6.2f +/- %0.2f Gyr' % (output_results[15], output_results[16])
    msg += '\n'
    msg += r'$\left< \log {\rm Z}^\star \right>_{\rm M}$ = %6.2f +/- %0.2f [Z/H]' % (output_results[17], output_results[18])
    # msg += '\n'
    # msg += r'$\log {\rm M}/{\rm M}$ = %6.2f' % output_results[-2]

    ax_lw.text(0.02, 0.9, msg, transform=ax_lw.transAxes, ha='left', va='top', fontsize=13)

    in_spec = spec__tw[0][wlmsk]
    out_spec = spec__tw[1][wlmsk]
    out_spec_joint = spec__tw[2][wlmsk]

    res = in_spec - out_spec
    wl = wl[wlmsk]
    ax_sp.set_ylabel('Flux [$10^{-16}$ erg/s/cm$^2$]')
    ax_sp.set_xlabel('wavelength [\AA]')
    ax_sp.plot(wl, in_spec, '-k', lw=3)
    ax_sp.plot(wl, out_spec, '-y', lw=0.7)
    ax_sp.plot(wl, (in_spec - out_spec), '-r', lw=1)

    if masks is not None:
        _ = [ax_sp.axvspan(msk[0], msk[1], alpha=0.3, color='gray') for msk in masks if msk[0] < wl_range[1]]
    if e_masks is not None:
        for msk in e_masks:
            l, r = msk
            if r > wl_range[1]:
                r = wl_range[1]
            if l > wl_range[0]:
                ax_sp.axvspan(l, r, alpha=0.2, color='blue')

    # Plot EL
    _ = [ax_sp.axvline(x=wlc, color='k', ls='--') for wlc in el_wl_fit]

    # COEFFS HEATMAP
    pwei, met_weights, age_weights = calc_weights(coeffs)
    pwei = pwei.T
    pwei.sort_index(axis='index', ascending=False, inplace=True)
    sns.heatmap((pwei / pwei.values.sum() * 100 if percent else pwei), cmap=cmap,
                center=None, vmin=vmin, vmax=vmax, square=False, linewidths=1.0,
                linecolor='w', annot=True, fmt=fmt, annot_kws={'fontsize':12},
                cbar=False, ax=ax,
                xticklabels=list(map(lambda v: '$%5.2f$'%v, np.log10(pwei.columns*1e9))),
                yticklabels=list(map(lambda v: '$%5.2f$'%v, pwei.index)))

    ax.set_xlabel(r'$\log {\rm t}^\star$ [yr]')
    ax.set_ylabel(r'$\log {\rm Z}^\star$ [${\rm Z}/{\rm Z}_\odot$]')
    ax_sp.set_xlim(wl_range)
    f.tight_layout()
    return f

def calc_weights(df):
    """Return the pivot matrix with coefficients and the
    marginal weights in age and metallicity"""
    coeffs = df.pivot_table(index="AGE", columns="MET", values="COEFF")

    met = coeffs.columns.values

    met_weights = coeffs / coeffs.sum(axis="columns", skipna=True).values[:,None]
    met_weights = met_weights.values[:,:,None].transpose(0,2,1)

    age_weights = coeffs.sum(axis="columns", skipna=True)
    age_weights /= np.nansum(age_weights)
    age_weights = age_weights.values[None,:]

    return coeffs, met_weights, age_weights

def read_masks(mask_list=None, elines_mask_file=None, redshift=0, sigma_mean=1,
               eline_half_range=None, verbose=0):
    masks = None
    e_w = None
    e_masks = None
    if mask_list is not None:
        if isfile(mask_list):
            masks = np.loadtxt(mask_list)
        else:
            print(f'{basename(sys.argv[0])}: {mask_list}: mask list file not found')
            mask_list = None
    else:
        print(f'{basename(sys.argv[0])}: no mask list file')
        # read elines mask
    if elines_mask_file is not None:
        if isfile(elines_mask_file):
            e_w = np.loadtxt(elines_mask_file, usecols=(0))
        else:
            print(f'{basename(sys.argv[0])}: {elines_mask_file}: emission lines mask file not found')
            elines_mask_file = None
    else:
        print(f'{basename(sys.argv[0])}: no elines mask file')

    z_fact = 1 + redshift
    eline_half_range = __mask_elines_window__ if eline_half_range is None else eline_half_range
    if e_w is not None:
        e_masks = np.empty((e_w.size, 2), dtype=np.float)
        eline_half_range *= sigma_mean
        if eline_half_range < 4:
            eline_half_range = 4
        e_masks[:,0] = e_w*z_fact - eline_half_range
        e_masks[:,1] = e_w*z_fact + eline_half_range
        e_masks = e_masks
        print_verbose(f'- Update emission line masks: ', verbose=verbose, level=2)
        print_verbose(e_masks, verbose=verbose, level=2)
        # if needed a complete mask:
    return masks, e_masks

def main(args):
    # read spec
    spec__tw, h = get_data_from_fits(args.out_file_fit, header=True)
    wl = get_wave_from_header(h)
    # read props
    props = np.loadtxt(args.out_file, delimiter=',', unpack=True)
    # read coeffs
    df = pd.read_csv(args.out_file_coeffs, sep='\t', usecols=[0, 1, 2, 3],
                     comment='#', names=['ID', 'AGE', 'MET', 'COEFF'],
                     header=None)
    idx = np.lexsort((df.MET, df.AGE))
    df = df.iloc[idx]
    df.MET = df.MET.apply(lambda x: np.log10(x/__solar_metallicity__))
    coeffs = df.reset_index(drop=True)

    if args.instrumental_dispersion is not None:
        sigma_unit = 'km/s'
        sigma_mean = np.sqrt(args.instrumental_dispersion**2 + (5000*props[9]/__c__)**2)
    else:
        sigma_unit = r'\AA'
        sigma_mean = props[9]

    # # generate list of the central wavelengths of fitted emission lines
    el_wl_fit = []
    # cf = ConfigAutoSSP(args.config_file)
    # for elcf in cf.systems_config:
    #     n_wl_fit = np.sum([
    #         elcf.check_par_fit('eline', a) for a in range(len(_MODELS_ELINE_PAR))
    #     ])
    #     for i, mt in enumerate(elcf.model_types):
    #         if ((mt == 'eline') and n_wl_fit):
    #             el_wl_fit.append(elcf.guess[i][_MODELS_ELINE_PAR['central_wavelength']]*(1+props[7]))
    if isfile(args.out_file_elines):
        _, _, output_systems = read_fit_elines_output(args.out_file_elines)
        el_wl_fit = sorted(itertools.chain(*[_s['central_wavelength'] for _s in output_systems]))

    # create masks ranges
    masks, e_masks = read_masks(mask_list=args.mask_list,
                                elines_mask_file=args.elines_mask_file,
                                redshift=props[7], sigma_mean=sigma_mean,
                                verbose=args.verbose)

    f = plot_output_resume(wl, spec__tw, props, coeffs, masks, e_masks, el_wl_fit,
                           wl_range=args.wl_range, cmap=args.cmap, vmin=0.1, vmax=50,
                           percent=not args.fraction)
    f.savefig(f'ana_spec_{basename(args.out_file)}.{args.img_suffix}', dpi=args.dpi)
    plt.show(block=args.block)
    plt.close(f)

if __name__ == '__main__':
    main(parse_arguments())



#!/usr/bin/env python3
# System imports
import sys
from os.path import basename

# Local imports
from pyFIT3D.common.io import ReadArguments
from pyFIT3D.common.io import call_cmnd

class ReadArgumentsLocal(ReadArguments):
    """
    Argument parser for this script
    """
    __script_name__ = basename(sys.argv[0])
    __mandatory__ = ['command']
    __optional__ = ['logfile', 'verbose']
    __def_optional__ = {'verbose': True}
    __arg_names__ = __mandatory__ + __optional__
    __N_tot_args__ = len(__arg_names__)
    __conv_func_mandatory__ = {'command': str}
    __conv_func_optional__ = {'logfile': str, 'verbose': bool}
    __conv_func__ = __conv_func_mandatory__.copy()
    __conv_func__.update(__conv_func_optional__)

    usage_msg_tmp1 = 'USE: {}'.format(__script_name__)
    usage_msg_tmp2 = ' "COMMAND" [LOGFILE] [VERBOSE=0/1]'
    __usage_msg__ = usage_msg_tmp1 + usage_msg_tmp2

    def __init__(self, args_list=None, verbose=False):
        ReadArguments.__init__(self, args_list, verbose=verbose)
        # self._parse_filenames()

if __name__ == '__main__':
    # pa stands for parsed arguments
    pa = ReadArgumentsLocal()
    call_cmnd(pa.command, pa.logfile, pa.verbose)



import numpy as np
from copy import deepcopy as copy
from scipy.signal import medfilt
from scipy.ndimage import convolve1d
from scipy.interpolate import interp1d

# local imports
from .io import print_verbose
from .constants import __c__, __shift_convolve_lnwl_frac__, __sigma_to_FWHM__

_STATS_POS = {
    'mean': 0,
    'pRMS': 1,
    'median': 2,
    'min': 3,
    'max': 4,
    'adev': 5,
    'RMS': 6,
}

def pdl_stats(x, ddof=1, bad=np.nan):
    # should return ($mean,$prms,$median,$min,$max,$adev,$rms)
    # MEAN = sum (x)/ N
    # PRMS = sqrt( sum( (x-mean(x))^2 )/(N-1))
    # AADEV = sum( abs(x-mean(x)) )/N
    # RMS = sqrt(sum( (x-mean(x))^2 )/N)
    [xmean, xpRMS, xmedian, xmin, xmax, xadev, xRMS] = [bad]*7
    if np.isfinite(x).any():
        x = x[np.isfinite(x)]
        N = x.size
        Np = N - ddof
        xmean = x.mean()
        d = x-xmean
        sdd = (d**2).sum()
        xRMS = np.sqrt(sdd/N)
        xpRMS = np.sqrt(sdd/Np)
        xmedian = np.median(x)
        xmin = np.min(x)
        xmax = np.max(x)
        xadev = np.sum(np.abs(d))/N
    return [xmean, xpRMS, xmedian, xmin, xmax, xadev, xRMS]


def WLS_invmat(y_obs, y_mod__m, dy=None):
    """
    Fit `y_obs` with linear combination of `y_mod__m` using Weighted
    Least Squares (WLS). An error `dy` (e.g. 1/sigma^2) could be set.

    TODO: Math stuff...

    Parameters
    ----------
    y_obs : array like
        Data to be fitted.
    y_mod__m : array like
        Set of models to fit `y_obs`.
    dy : array like, optional
        Error of `y_obs`. Default is None.

    Returns
    -------
    array like
        Best model of `y_obs`.
    array like
        Best coefficients of `y_mod__m`.
    """
    A = np.asarray(y_mod__m)
    y_obs_mean = y_obs.mean()
    B = y_obs/y_obs_mean
    # from sklearn.linear_model import LinearRegression
    # WLS = LinearRegression()
    # print(A.T, B.shape, dy.shape)
    # WLS.fit(A.T, B, sample_weight=dy)
    # coeffs = WLS.coef_[0]*y_obs_mean
    # return A*coeffs, coeffs
    # XXX: This function tries to bypass the Singular Matrix
    # problem inverting the matrix using pinv which computes
    # the (Moore-Penrose) pseudo-inverse of a matrix
    if dy is not None:
        AwAT = np.dot(A, np.diag(dy).dot(A.T))
        try:
            AwAT_inv = np.linalg.inv(AwAT)
        except np.linalg.LinAlgError:
            AwAT_inv = np.linalg.pinv(AwAT)
        AwB = A.dot(np.diag(dy).dot(B))
        p = np.dot(AwAT_inv, AwB)
    else:
        AAT = A.dot(A.T)
        try:
            AAT_inv = np.linalg.inv(AAT)
        except np.linalg.LinAlgError:
            AAT_inv = np.linalg.pinv(AAT)
        p = AAT_inv.dot(A).dot(B)
    p *= y_obs_mean
    return A.T.dot(p), p

# Hector median_filter copy of SFS median_filter in perl
def median_filter(box, x, verbose=False):
    """ Apply a median filter to `x` with box size `box`.

    Parameters
    ----------
    box: int
        Box size of the median filter.
    x: array like
        Array to be filtered.

    Returns
    -------
    array like
        `x` input array filtered
    """
    x = np.asarray(x)
    box_size = 2*box
    box_size = round_up_to_odd(box_size)
    if box_size >= x.size:
        print_verbose(f'[median_filter]: box_size ({box_size}) greater than x.size ({x.size}).', verbose=verbose)
        return x
    val = copy(x)
    for i in range(box, val.size - box):
        val[i] = np.median([x[i - box + j] for j in range(0, 2*box)])
    for i in range(1, box):
        val[i] = np.median([x[j] for j in range(0, 2*i)])
    for i in range(val.size - box, val.size - 1):
        val[i] = np.median([x[i - (val.size - i) + j] for j in range(0, 2*(val.size - i))])
    return val

def round_up_to_odd(n):
    """ Rounds up `n` to the next odd integer.

    Parameters
    ----------
    n : float
        A number to be rounded to the next odd integer.

    Returns
    -------
    int
        `n` rounded to the next odd integer.
    """
    n = int(np.ceil(n))
    return n + 1 if n % 2 == 0 else n

def smooth_ratio(flux_ratio, sigma, kernel_size_factor=None):
    """ Create a smooth factor using the ratio r = `flux_a`/`flux_b`
    through a median filter.

    Parameters
    ----------
    flux_ratio : array like
        The flux_ratio which will be passed through a median_filter
    sigma : float
        Sigma in angstroms.
    kernel_size_factor : float
        Will define, together with `sigma`, the kernel_size.

        kernel_size = next odd integer from int(kernel_size_factor * sigma)

    Returns
    -------
    array like
        The smooth ratio.
    """
    if sigma < 1:
        sigma = 1

    if kernel_size_factor is None:
        kernel_size_factor = 7*__sigma_to_FWHM__

    # setbadtoval(1)
    flux_ratio[~(np.isfinite(flux_ratio))] = 1

    kernel_size = round_up_to_odd(np.int(kernel_size_factor*sigma))
    # return median_filter(kernel_size, flux_ratio)
    sm_ratio = median_filter(kernel_size, flux_ratio)
    # if (sm_ratio == flux_ratio).all():
    #     sm_ratio = np.ones_like(flux_ratio)
    sm_ratio = np.clip(sm_ratio, 0, None)
    # sm_ratio = np.clip(sm_ratio, 0, 1)
    sm_ratio[~(np.isfinite(sm_ratio))] = 1
    return sm_ratio
    # return medfilt(flux_ratio, kernel_size)

def convolve_sigma(flux, sigma, side_box=None):
    """
    Convolves `flux` using a Gaussian-kernel with standard deviation `sigma`.
    The kernel have dimension 2*`side_box` + 1.

    Parameters
    ----------
    flux : array like
        Spectrum to be convolved.
    sigma : float
        Sigma of the Gaussian-kernel.
    N_side: float
        Will define the range size of the Gaussian-kernel.

    Returns
    -------
    array like
        Convolved `flux` by the weights defined by the Gaussian-kernel.
    """
    kernel_function = lambda x: np.exp(-0.5*(((x - side_box)/sigma)**2))
    N = 2*side_box + 1
    kernel = np.array(list(map(kernel_function, np.arange(N))))
    norm = kernel.sum()
    kernel = kernel/norm
    return convolve1d(flux, kernel, mode='nearest')

def shift_convolve(wave_obs, wave_in, flux_in, redshift, sigma, sigma_inst=None):
    """
    Shift and convolve spectrum.

    Shift the spectrum `flux_in` at `wave_obs` wavelenghts to `wave_in`
    corrected in redshift. Also convolves the spectrum to `sigma` +
    `sigma_inst`. If `sigma_inst` is None, the shift + convolution of
    `flux_in` is simplier and faster.


    Parameters
    ----------
    wave_obs : array like
        Observed wavelenghts.

    wave_in : array like
        Input wavelengts of SSP models at observed frame.

    flux_in : array like
        Flux in `wave_obs` wavelenghts.

    redshift : float
        Input redshift.

    sigma : float
        Velocity dispersion of data in km/s.

    sigma_inst : float, optional
        Instrumental velocity dispersion in Angstrom.
        Defaults to None.

    Returns
    -------
    array like
        The `flux_in` convolved by `sigma` + `sigma_int` and
        shifted (interpoled) to `wave_in` wavelenghts.

    See also
    --------
    `pyFIT3D.common.stats.convolve_sigma`, `scipy.interpolate.interp1d`
    """
    def conv_interp(w_out, w_in, f_in, sigma=None, box=0, conv=True):
        f = interp1d(
            w_in,
            convolve_sigma(f_in, sigma, 3 if box < 3 else box) if conv else f_in,
            assume_sorted=True, kind='linear', bounds_error=False, fill_value='extrapolate'
        )
        return f(w_out)

    # bring everything to the observed frame
    wave_in_of = wave_in*(1 + redshift)
    dpix_ini = wave_in_of[1] - wave_in_of[0]
    # in the case that the instrumental sigma is not avaible,
    # the shift+convolution process is simplier.
    if sigma_inst is None:
        rsigma = sigma/dpix_ini
        return conv_interp(wave_obs, wave_in_of, flux_in, rsigma, int(3*rsigma))

    dpix = dpix_ini/(1 + np.random.rand())
    # dpix = dpix_ini
    # interpolate flux_in to the new wavelenghts.
    w_min = wave_in_of.min()
    w_max = wave_in_of.max()
    N_sub = (w_max - w_min)/dpix
    wave = w_min + dpix*np.arange(N_sub)
    # resample wave in ln(wavelenght)
    ln_wave = np.log(wave)
    # interpolation factor
    f_fine = __shift_convolve_lnwl_frac__
    ln_dpix = (ln_wave[1] - ln_wave[0])/f_fine
    ln_w_min = ln_wave.min()
    ln_w_max = ln_wave.max()
    N_ln_wave = (ln_w_max - ln_w_min)/ln_dpix
    new_ln_wave = ln_w_min + ln_dpix*np.arange(N_ln_wave)
    # instrumental resolution in pixels
    rsigma_inst = sigma_inst/dpix
    new_wave_inst = np.exp(new_ln_wave)
    #########################################
    # Shift + convolve sigma_inst and sigma #
    #########################################
    # interpolated and sigma_inst convolved flux_in convolved to sigma
    return conv_interp(
            w_out=wave_obs, w_in=new_wave_inst,

            # interpolated flux_in convolved to sigma_inst.
            f_in=conv_interp(
                w_out=new_wave_inst, w_in=wave,

                # interpolate flux_in to the new wavelenghts.
                f_in=conv_interp(w_out=wave, w_in=wave_in_of, f_in=flux_in, conv=False),

                sigma=rsigma_inst, box=int(6*rsigma_inst), conv=(rsigma_inst > 0.5)
            ),

            sigma=((1 if sigma == 0 else sigma)/__c__)/ln_dpix,
            box=int(5*(500/__c__)/ln_dpix), conv=True,
        )

def calc_chi_sq(f_obs, f_mod, ef_obs, ddof=0):
    """
    Calculates the Chi Square of a fitted model.

    Parameters
    ----------
    f_obs : array like
        Observed spectrum
    f_mod : array like
        Modeled spectrum
    ef_obs : array like
        Error of observed spectrum.

    Returns
    -------
    float
        The Chi Square of the fit.
    int
        The number of observations.
    """
    mask = ef_obs != 0
    N_obs = mask.sum()
    chi = np.divide(
        f_obs - f_mod, ef_obs,
        where=mask,
        out=np.zeros_like(f_obs)
    )
    chi_sq = np.sum(chi**2)
    chi_sq_red = ((chi_sq / (N_obs - ddof)) if N_obs - ddof > 0 else chi_sq)
    return chi_sq_red, N_obs

def hyperbolic_fit_par(x, y, verbose=False):
    """
    Calculates the parameter `x` considering `y` with an hyperbolic model function fit.
    This function assumes that the three last values of x and y are forming an hyperbole.

    Parameters
    ----------
    x : array like
        The x values.
    y : array like
        The y(x) values.
    verbose : bool, optional
        Print output errors. Default is False.
    """
    par = None
    error_par = None
    if len(y) > 2:
        a, b, c = x[-3:]
        fa, fb, fc = y[-3:]
        delta_ba = b - a
        delta_fba = fb - fa
        delta_bc = c - b
        delta_fcb = fc - fb
        den = fa + fc - 2*fb
        if (den != 0):
            par = c - delta_ba*(delta_fcb/den + 0.5)
        else:
            par = 0
        slope = np.abs(0.5*delta_fcb/delta_bc) + np.abs(0.5*delta_fba/delta_ba)
        if slope != 0:
            error_par = 0.01 * par/slope
    else:
        print_verbose(f'[hyperbolic_fit: n={len(y)}] Impossible to calculate hyperbolic fit!', verbose=verbose)
    return par, error_par

def median_box(box_init, x):
    """
    Creates a box with spans the same range of `x` evenly spaced with a median box.
    The box size is 2*`box_init`. If `box_init` is even, it will be rewritten to the
    next odd integer.

    Parameters
    ----------
    box_init : int
        The pixel where the box begins.
    x : array like
        The array which will be 'boxed'.

    Returns
    -------
    array like
        Evenly spaced box with approximately the same range of `x`.
    """
    box_init = round_up_to_odd(box_init)
    end_box = len(x) - box_init
    box_size = 2*box_init
    xbox = np.array([
        np.median([x[i - box_init + j] for j in range(box_size)])
        for i in range(box_init, end_box, box_size)
    ])
    return xbox

def std_m(data):
    mean = np.median(data)
    sum_res = 0
    if len(data) > 0:
        for dat in data:
            sum_res += (dat - mean)**2
        sum_res /= (len(data)-1)
        result = np.sqrt(sum_res)
    else:
        result = 0
    return result



#!/usr/bin/env python3
################################################################################
#
# pack_NAME.py
# ----------------------
# Perl script: pack_NAME.pl
#
################################################################################

# System imports
import sys
from os.path import basename

# Local imports
from pyFIT3D.common.io import ReadArguments
from pyFIT3D.common.tools import pack_NAME

class ReadArgumentsLocal(ReadArguments):
    """
    Argument parser for this script
    """
    __script_name__ = basename(sys.argv[0])
    __mandatory__ = ['name']
    __optional__ = ['ssp_pack_filename', 'elines_pack_filename', 'sfh_pack_filename']
    __def_optional__ = {
        'ssp_pack_filename': '../legacy/pack_CS_inst_disp.csv',
        'elines_pack_filename': '../legacy/pack_elines_v1.5.csv',
        'sfh_pack_filename': '../legacy/pack_SFH.csv'
    }
    __arg_names__ = __mandatory__ + __optional__
    __N_tot_args__ = len(__arg_names__)
    __conv_func_mandatory__ = {'name': str}
    __conv_func_optional__ = {'ssp_pack_filename': str, 'elines_pack_filename': str, 'sfh_pack_filename': str}
    __conv_func__ = __conv_func_mandatory__.copy()
    __conv_func__.update(__conv_func_optional__)

    usage_msg_tmp1 = 'USE: {}'.format(__script_name__)
    usage_msg_tmp2 = ' NAME [SSP_PACK] [ELINES_PACK] [SFH_PACK]'
    __usage_msg__ = usage_msg_tmp1 + usage_msg_tmp2

    def __init__(self, args_list=None, verbose=False):
        ReadArguments.__init__(self, args_list, verbose=verbose)
        # self._parse_filenames()

if __name__ == '__main__':
    # pa stands for parsed arguments
    pa = ReadArgumentsLocal()
    pack_NAME(pa.name, pa.ssp_pack_filename, pa.elines_pack_filename, pa.sfh_pack_filename)



#!/usr/bin/env python3
import sys
import numpy as np
from os.path import basename, isfile
from pyFIT3D.common.io import ReadArguments
from pyFIT3D.common.gas_tools import kin_rss_elines

np.set_printoptions(precision=4, suppress=True, linewidth=200)

class ReadArgumentsLocal(ReadArguments):
    """
    Argument parser for script that fits the emission-lines of a RSS FITS.

    To add an argument to the script:
        Argument name in `__mandatory__` or `__optional__` list.
        Argument conversion function in `__mandatory_conv_func___` or `__optional_conv_func___` list.
        Argument default value (if not mandatory) in `__def_optional__`
    """
    # class static configuration:
    # arguments names and conversion string to number functions
    __script_name__ = basename(sys.argv[0])
    __mandatory__ = ['spec_file', 'config_file', 'mask_file', 'w_min', 'w_max', 'out_file']
    __optional__ = ['n_MC', 'n_loops', 'plot', 'scale_ini', 'guided', 'memo', 'prefix']
    __arg_names__ = __mandatory__ + __optional__
    __N_tot_args__ = len(__arg_names__)
    # default values of optional arguments with __optional__ as keys
    __def_optional__ = {
        'n_MC': 50, 'n_loops': 15, 'plot': 0, 'scale_ini': 0.15, 'guided': 0, 'memo': 0,
        # 'out_file': 'out.fit_spectra',
        # 'out_mod_res_final': __script_name__.replace('.py', '.out')
    }

    # parse functions
    __conv_func_mandatory__ = {'spec_file': str, 'config_file': str, 'mask_file': str, 'out_file': str}
    __conv_func_optional__ = {'prefix': str}
    __conv_func__ = __conv_func_mandatory__.copy()
    __conv_func__.update(__conv_func_optional__)

    # usage message
    __usage_msg__ = 'usage: {} SPECTRUM_FILE[,ERROR_FILE] CONFIG MASK_LIST W_MIN W_MAX OUTFILE'.format(__script_name__)
    __usage_msg__ += ' [N_MC={:d}]'.format(__def_optional__['n_MC'])
    __usage_msg__ += ' [N_LOOPS={:d}]]'.format(__def_optional__['n_loops'])
    __usage_msg__ += ' [PLOT] [SCALE_INI={:.2f}]'.format(__def_optional__['scale_ini'])
    __usage_msg__ += ' [guided] [MEMO=0/1]'
    # __usage_msg__ += ' [OUTPUT_FILENAME] [OUTPUT_MOD_RES_FILENAME]'

    def __init__(self, args_list=None, verbose=False):
        ReadArguments.__init__(self, args_list, verbose=verbose)
        self.redefine_max = 0
        self._parse_filenames()

    def _parse_filenames(self):
        self.error_file = None
        spec_file_args = self.spec_file.split(',')
        if len(spec_file_args) == 2:
            self.spec_file = spec_file_args[0]
            self.error_file = spec_file_args[1]

if __name__ == "__main__":
    # parse arguments
    pa = ReadArgumentsLocal()
    kin_rss_elines(spec_file=pa.spec_file, config_file=pa.config_file, guided=pa.guided,
                   w_min=pa.w_min, w_max=pa.w_max, out_file=pa.out_file, mask_file=pa.mask_file,
                   error_file=pa.error_file, n_MC=pa.n_MC, n_loops=pa.n_loops, plot=pa.plot,
                   scale_ini=pa.scale_ini, memo=pa.memo, run_mode='LM',
                   prefix=pa.prefix, seed=None)



#!/usr/bin/env python3
################################################################################
#
# redshift_config.py
# ----------------------
# Perl script: redshift_config.pl
#
################################################################################

# System imports
import sys
from os.path import basename

# Local imports
from pyFIT3D.common.io import ReadArguments
from pyFIT3D.common.tools import redshift_config

class ReadArgumentsLocal(ReadArguments):
    """
    Argument parser for this script
    """
    __script_name__ = basename(sys.argv[0])
    __mandatory__ = ['input_config', 'redshift', 'output_config']
    __arg_names__ = __mandatory__
    __N_tot_args__ = len(__arg_names__)
    __conv_func_mandatory__ = {'input_config': str, 'output_config': str}
    __conv_func__ = __conv_func_mandatory__.copy()

    usage_msg_tmp1 = 'USE: {}'.format(__script_name__)
    usage_msg_tmp2 = ' input_config redshift output_config'
    __usage_msg__ = usage_msg_tmp1 + usage_msg_tmp2

    def __init__(self, args_list=None, verbose=False):
        ReadArguments.__init__(self, args_list, verbose=verbose)
        # self._parse_filenames()

if __name__ == '__main__':
    # pa stands for parsed arguments
    pa = ReadArgumentsLocal()
    redshift_config(pa.input_config, pa.redshift, pa.output_config)



#!/usr/bin/env python3
import sys
import numpy as np
from os.path import join, basename
from copy import deepcopy as copy

from pyFIT3D.common.constants import __c__
from pyFIT3D.common.tools import radial_sum_cube_e
from pyFIT3D.common.io import ReadArguments, get_data_from_fits, get_wave_from_header, output_spectra

class ReadArgumentsLocal(ReadArguments):
    """
    Argument parser for the emission-lines fit scripts.

    To add an argument to the script:
        Argument name in `__mandatory__` or `__optional__` list.
        Argument conversion function in `__mandatory_conv_func___` or `__optional_conv_func___` list.
        Argument default value (if not mandatory) in `__def_optional__`
    """
    # class static configuration:
    # arguments names and conversion string to number functions
    __script_name__ = basename(sys.argv[0])
    __mandatory__ = ['cube_filename', 'name', 'x0', 'y0', 'output_path']
    __optional__ = ['delta_R', 'badpix_ext', 'badpix_val', 'badpix_coll_threshold_frac']
    __arg_names__ = __mandatory__ + __optional__
    __N_tot_args__ = len(__arg_names__)
    # default values of optional arguments with __optional__ as keys
    __def_optional__ = {
        'delta_R': 2.5, 'badpix_ext': 3, 'badpix_val': 1, 'badpix_coll_threshold_frac': 0.75
    }

    # parse functions
    __conv_func_mandatory__ = {'cube_filename': str, 'name': str, 'output_path': str}
    __conv_func_optional__ = {'badpix_ext': int}
    __conv_func__ = __conv_func_mandatory__.copy()
    __conv_func__.update(__conv_func_optional__)

    # usage message
    __usage_msg__ = 'usage: {} FILENAME NAME X0 Y0 OUTPUT_PATH '.format(__script_name__)
    __usage_msg__ += ' [DELTA_R={}]'.format(__def_optional__['delta_R'])
    __usage_msg__ += ' [BADPIX_EXT={}]'.format(__def_optional__['badpix_ext'])
    __usage_msg__ += ' [BADPIX_VAL={}]'.format(__def_optional__['badpix_val'])
    __usage_msg__ += ' [BADPIX_COLL_THRESHOLD_FRACTION={}]'.format(__def_optional__['badpix_coll_threshold_frac'])

    def __init__(self, args_list=None, verbose=False):
        ReadArguments.__init__(self, args_list, verbose=verbose)

def collapse_badpixels_mask(mask__wyx, threshold_fraction=1, mask_value=1):
    """
    Performs the bad pixels mask collapse, i.e., generates a 2D map from a 3D
    (wavelengths, y, x) masking spaxels with a fraction of masked pixels equal
    or above `threshold`.

    Parameters
    ----------
    mask__wyx : array like
        Bad pixels mask cube (3 dimensions, NWAVE, NY, NX).

    threshold_fraction : float
        Sets the threshold to be considered a bad spaxel, i.e. spaxels with a
        fraction of masked pixels equal or above this threshold will be masked.
        threshold = NWAVE * threshold_fraction.

    Returns
    -------
    array like
        2D bad spaxels map.
    """
    nw, ny, nx = mask__wyx.shape
    threshold = nw*threshold_fraction
    badspaxels__yx = np.zeros((ny, nx), dtype=mask__wyx.dtype)
    badpixels_total__yx = mask__wyx.sum(axis=0)
    badspaxels__yx[badpixels_total__yx/nw >= threshold] == 1
    return badspaxels__yx

if __name__ == '__main__':
    pa = ReadArgumentsLocal()

    # Central and integrated spec extraction
    org_cube__wyx, org_h, n_ext = get_data_from_fits(pa.cube_filename, header=True, return_n_extensions=True)
    org_wave__w = get_wave_from_header(org_h, wave_axis=3)
    h_set_rss = {'CRVAL1': org_h['CRVAL3'], 'CDELT1': org_h['CDELT3'], 'CRPIX1': org_h['CRPIX3']}
    h_set_cube = {'CRVAL3': org_h['CRVAL3'], 'CDELT3': org_h['CDELT3'], 'CRPIX3': org_h['CRPIX3']}
    med_vel = org_h.get('MED_VEL')
    org_badpix__wyx = None
    org_error__wyx = None
    org_badspaxels__yx = None
    if org_h['EXTEND']:
        org_error__wyx = get_data_from_fits(pa.cube_filename, extension=1)
        org_badpix__wyx = get_data_from_fits(pa.cube_filename, extension=pa.badpix_ext)
        if pa.badpix_val != 1:
            _tmp_org_badpix__wyx = np.ones_like(org_badpix__wyx)
            _tmp_org_badpix__wyx[org_badpix__wyx != pa.badpix_val] = 0
            org_badpix__wyx = _tmp_org_badpix__wyx
        org_badspaxels__yx = collapse_badpixels_mask(org_badpix__wyx,
                                                     threshold_fraction=pa.badpix_coll_threshold_frac,
                                                     mask_value=1)
    r = radial_sum_cube_e(copy(org_cube__wyx), x0=pa.x0, y0=pa.y0, delta_R=pa.delta_R,
                          input_mask=org_badpix__wyx, input_error=org_error__wyx)
    output__rw, e_output__rw, _ = r
    f_cen__w = copy(output__rw[0])
    f_cen__w[~np.isfinite(f_cen__w)] = 0
    sqrt_f_cen__w = np.sqrt(np.abs(f_cen__w))
    e_f_cen__w = copy(e_output__rw[0])
    e_f_cen__w[~np.isfinite(e_f_cen__w)] = 1e12
    e_f_cen__w = np.where(e_f_cen__w > sqrt_f_cen__w, sqrt_f_cen__w, e_f_cen__w)
    cen_spec_filename = f'{pa.name}.spec_5.txt'
    output_spectra(org_wave__w, [f_cen__w, e_f_cen__w], join(pa.output_path, cen_spec_filename))
    f_int__w = copy(output__rw[5])
    f_int__w[~np.isfinite(f_int__w)] = 0
    sqrt_f_int__w = np.sqrt(np.abs(f_int__w))
    e_f_int__w = copy(e_output__rw[5])
    e_f_int__w[~np.isfinite(e_f_int__w)] = 1e12
    e_f_int__w = np.where(e_f_int__w > sqrt_f_int__w, sqrt_f_int__w, e_f_int__w)
    int_spec_filename = f'{pa.name}.spec_30.txt'
    output_spectra(org_wave__w, [f_cen__w, e_f_cen__w], join(pa.output_path, int_spec_filename))



import sys
import numpy as np
from astropy.io import fits
from copy import deepcopy as copy
from os.path import basename, isfile

from pyFIT3D.modelling.stellar import StPopSynt

from .io import read_spectra, print_time, clean_preview_results_files, ReadArguments
from .io import array_to_fits, trim_waves, sel_waves, print_verbose, write_img_header
from .gas_tools import append_emission_lines_parameters
from .gas_tools import ConfigEmissionModel, create_emission_lines_parameters

from .constants import __selected_extlaw__, __selected_R_V__, __n_Monte_Carlo__

__verbose__ = True
__verbose__ = False

class ConfigAutoSSP(object):
    """Reads, stores and process the configuration of AutoSSP script. Also, load masks and
    SSP models.
    This class

    Attributes
    ----------
    args : ReadArgumentsAutoSSP class
    filename :

    Methods
    -------
    _load :
    _load_ssp_fits :
    _multi_AV :
    _load_masks :

    """

    def __init__(self, config_file,
                 redshift_set=None, sigma_set=None, AV_set=None,
                 w_min=None, w_max=None, nl_w_min=None, nl_w_max=None,
                 mask_list=None, elines_mask_file=None,
                 sigma_inst=None, verbose=False):
        self.filename = config_file
        self.redshift_set = redshift_set
        self.sigma_set = sigma_set
        self.AV_set = AV_set
        self._verbose = verbose
        self._load()

    def _load(self):
        """
        Loads the configuration file. Also, reads the configuration file of
        each to-be-fitted system.
        """
        config_keys = [
            'redshift', 'delta_redshift','min_redshift','max_redshift',
            'DV','RV','DS','RS','MIN_W','MAX_W',
            'sigma','delta_sigma','min_sigma','max_sigma',
            'AV','delta_AV','min_AV','max_AV',
        ]
        self.systems = []
        self.n_systems = 0
        # Array of `ConfigEmissionModel`
        self.systems_config = []
        self.start_w = 1e12
        self.end_w = -1e12

        with open(self.filename, "r") as f:
            # get file size
            f.seek(0, 2)
            file_size = f.tell()
            f.seek(0)

            # reads each line till systems config
            for k, v in zip(config_keys[0:10], f.readline().split()):
                setattr(self, k, eval(v))
            for k, v in zip(config_keys[10:14], f.readline().split()):
                setattr(self, k, eval(v))
            for k, v in zip(config_keys[14:18], f.readline().split()):
                setattr(self, k, eval(v))
            self.n_systems = eval(f.readline())

            # redefine parameters setted by user
            if self.redshift_set is not None:
                self.redshift = self.redshift if self.redshift_set[0] is None else self.redshift_set[0]
                self.delta_redshift = self.delta_redshift if self.redshift_set[1] is None else self.redshift_set[1]
                self.min_redshift = self.min_redshift if self.redshift_set[2] is None else self.redshift_set[2]
                self.max_redshift = self.max_redshift if self.redshift_set[3] is None else self.redshift_set[3]
            if self.sigma_set is not None:
                self.sigma = self.sigma if self.sigma_set[0] is None else self.sigma_set[0]
                self.delta_sigma = self.delta_sigma if self.sigma_set[1] is None else self.sigma_set[1]
                self.min_sigma = self.min_sigma if self.sigma_set[2] is None else self.sigma_set[2]
                self.max_sigma = self.max_sigma if self.sigma_set[3] is None else self.sigma_set[3]
            if self.AV_set is not None:
                self.AV = self.AV if self.AV_set[0] is None else self.AV_set[0]
                self.delta_AV = self.delta_AV if self.AV_set[1] is None else self.AV_set[1]
                self.min_AV = self.min_AV if self.AV_set[2] is None else self.AV_set[2]
                self.max_AV = self.max_AV if self.AV_set[3] is None else self.AV_set[3]

            # reads each system config
            for i in range(self.n_systems):
                l = f.readline().split()
                tmp = {
                    'start_w': eval(l[0]), 'end_w': eval(l[1]), 'mask_file': l[2],
                    'config_file': l[3], 'npoly': eval(l[4]),
                    'mask_poly': l[5], 'nmin': eval(l[6]), 'nmax': eval(l[7])
                }
                if tmp['start_w'] < self.start_w:
                    self.start_w = tmp['start_w']
                if tmp['end_w'] > self.end_w:
                    self.end_w = tmp['end_w']
                if not isfile(tmp['mask_file']):
                    tmp['mask_file'] = None
                self.systems.append(tmp)
                self.systems_config.append(ConfigEmissionModel(tmp['config_file'], verbose=self._verbose))
            l = f.readline().split()
            self.MIN_DELTA_CHI_SQ = eval(l[0])
            self.MAX_N_ITER = eval(l[1])
            self.CUT_MEDIAN_FLUX = eval(l[2])
            self.ABS_MIN = 0.5*self.CUT_MEDIAN_FLUX

            l = f.readline().split()
            self.start_w_peak = eval(l[0])
            self.end_w_peak = eval(l[1])

            # Some configuration files could have this line
            # Not tested yet
            if (f.tell() != file_size):
                l = f.readline().split()
                if len(l) > 0:
                    self.wave_norm = eval(l[0])
                    self.w_wave_norm = eval(l[1])
                    self.new_ssp_file = l[2]

        print_verbose(f'{self.n_systems} Number of systems', verbose=self._verbose)

        return None

def auto_ssp_elines_single_main(
    wavelength, flux, eflux, ssp_file, config_file, out_file,
    ssp_nl_fit_file=None, sigma_inst=None, mask_list=None, elines_mask_file=None,
    min=None, max=None, w_min=None, w_max=None, nl_w_min=None, nl_w_max=None,
    input_redshift=None, delta_redshift=None, min_redshift=None, max_redshift=None,
    input_sigma=None, delta_sigma=None, min_sigma=None, max_sigma=None,
    input_AV=None, delta_AV=None, min_AV=None, max_AV=None,
    R_V=None, extlaw=None, plot=None, single_ssp=False,
    is_guided_sigma=False,
    # guided_sigma=None,
    ratio=True, spec_id=None, y_ratio=None, guided_errors=None, fit_sigma_rnd=True,
    fit_gas=True, losvd_rnd_medres_merit=False, verbose=None, sps_class=StPopSynt):

    verbose = __verbose__ if verbose is None else verbose
    redshift_set = [input_redshift, delta_redshift, min_redshift, max_redshift]
    sigma_set = [input_sigma, delta_sigma, min_sigma, max_sigma]
    AV_set = [input_AV, delta_AV, min_AV, max_AV]

    cf = ConfigAutoSSP(config_file, redshift_set=redshift_set, sigma_set=sigma_set, AV_set=AV_set)
    if plot:
        import matplotlib.pyplot
        import seaborn as sns
        sns.set(context="paper", style="ticks", palette="colorblind", color_codes=True)
    if guided_errors is not None:
        fit_gas = False
    SPS = sps_class(config=cf,
                    wavelength=wavelength, flux=flux, eflux=eflux,
                    mask_list=mask_list, elines_mask_file=elines_mask_file,
                    sigma_inst=sigma_inst, ssp_file=ssp_file,
                    ssp_nl_fit_file=ssp_nl_fit_file, out_file=out_file,
                    w_min=w_min, w_max=w_max, nl_w_min=nl_w_min, nl_w_max=nl_w_max,
                    R_V=R_V, extlaw=extlaw, spec_id=spec_id, min=min, max=max,
                    guided_errors=guided_errors, ratio_master=y_ratio,
                    fit_gas=fit_gas, plot=plot, verbose=verbose)
    msg_cut = f' - cut value: {cf.CUT_MEDIAN_FLUX:6.4f}'
    if cf.CUT_MEDIAN_FLUX == 0:
        msg_cut = ' - Warning: no cut (CUT_MEDIAN_FLUX = 0)'
    print(f'-> median raw flux = {SPS.median_flux:6.4f}{msg_cut}')
    SPS.cut = False
    # valid_flux = 2 - everything OK
    # valid_flux = 1 - missing valid flux (flux > 0) inside nl-fit wavelength window.
    # valid_flux = 0 - missing valid flux (flux > 0) inside linear fit wavelength window
    if (SPS.valid_flux > 0) and (SPS.median_flux > cf.CUT_MEDIAN_FLUX):  # and (median_flux > cf.ABS_MIN):
        # redshift, sigma and AV fit
        SPS.non_linear_fit(is_guided_sigma, fit_sigma_rnd=fit_sigma_rnd,
                           sigma_rnd_medres_merit=losvd_rnd_medres_merit)
        # SPS.non_linear_fit(guided_sigma)
        SPS.calc_SN_norm_window()
        min_chi_sq = 1e12
        n_iter = 0
        while ((min_chi_sq > cf.MIN_DELTA_CHI_SQ) & (n_iter < cf.MAX_N_ITER)):
            print(f'Deriving SFH... attempt {n_iter + 1} of {cf.MAX_N_ITER}')

            # This part is repeated in auto_ssp perl script
            # if cf.CUT_MEDIAN_FLUX == 0:
            #     msg_cut = ' - Warning: no cut (CUT_MEDIAN_FLUX = 0)'
            # print(f'-> median masked flux = {med_flux:6.4f}{msg_cut}')
            # if SPS.med_flux >= cf.CUT_MEDIAN_FLUX:

            # Emission lines fit
            SPS.gas_fit(ratio=ratio)
            # stellar population synthesis based solely on chi_sq determination
            # if median_flux > cf.CUT_MEDIAN_FLUX:
            if single_ssp:
                min_chi_sq_now = SPS.ssp_single_fit()
            else:
                min_chi_sq_now = SPS.ssp_fit(n_MC=__n_Monte_Carlo__)
            SPS.resume_results()
            print(f'Deriving SFH... attempt {n_iter + 1} DONE!')
            if not single_ssp:
                SPS.output_coeffs_MC_to_screen()
                SPS.output_to_screen(block_plot=False)
            if min_chi_sq_now < min_chi_sq:
                min_chi_sq = min_chi_sq_now
            n_iter += 1
    else:
        SPS.cut = True
        # TODO: What to do when cut ?
        # SPS.output_gas_emission will fail because
        # SPS.config.systems[:]['EL'] it will not be defined.
        print('-> median flux below cut: unable to perform analysis.')
    return cf, SPS

# TODO: ADD SINGLE SSP FIT TO AUTO_SSP_SPEC
def auto_ssp_spec(
    spec_file, ssp_models_file, out_file, config_file,
    error_file=None, variance_error_column=False,
    nl_ssp_models_file=None, instrumental_dispersion=None,
    wl_range=None, nl_wl_range=None, mask_list=None, elines_mask_file=None,
    redshift_set=None, losvd_set=None, losvd_in_AA=False, AV_set=None,
    fit_gas=True, fit_sigma_rnd=True, single_ssp=False,
    plot=None, min=None, max=None, seed=None, ratio=None,
    losvd_rnd_medres_merit=False, verbose=0,
    R_V=None, extlaw=None,
    ):
    time_ini_run = print_time(print_seed=False, get_time_only=True)
    # initial time in seconds since Epoch.
    seed = print_time() if seed is None else print_time(time_ini=seed)
    # initial time used as the seed of the random number generator.
    np.random.seed(seed)
    ratio = True if ratio is None else ratio
    nl_ssp_models_file = ssp_models_file if nl_ssp_models_file is None else nl_ssp_models_file
    out_file_elines = 'elines_' + out_file
    out_file_single = 'single_' + out_file
    out_file_coeffs = 'coeffs_' + out_file
    out_file_fit = 'output.' + out_file + '.fits'
    out_file_ps = out_file
    w_min = None if wl_range is None else wl_range[0]
    w_max = None if wl_range is None else wl_range[1]
    nl_w_min = None if nl_wl_range is None else nl_wl_range[0]
    nl_w_max = None if nl_wl_range is None else nl_wl_range[1]
    if redshift_set is not None:
        input_redshift, delta_redshift, min_redshift, max_redshift = redshift_set
    else:
        input_redshift, delta_redshift, min_redshift, max_redshift = None, None, None, None
    if losvd_set is not None:
        input_sigma, delta_sigma, min_sigma, max_sigma = losvd_set
    else:
        input_sigma, delta_sigma, min_sigma, max_sigma = None, None, None, None
    if AV_set is not None:
        input_AV, delta_AV, min_AV, max_AV = AV_set
    else:
        input_AV, delta_AV, min_AV, max_AV = None, None, None, None

    # remove old files
    clean_preview_results_files(out_file, out_file_elines, out_file_single, out_file_coeffs, out_file_fit)

    # read spectrum
    wl__w, f__w, ef__w = read_spectra(spec_file, f_error=lambda x: 0.1*np.sqrt(np.abs(x)), variance_column=variance_error_column)

    input_SN = np.divide(f__w, ef__w, where=ef__w!=0)
    print_verbose(f'-> mean input S/N: {input_SN[np.isfinite(input_SN)].mean()}', verbose=verbose)

    cf, SPS = auto_ssp_elines_single_main(
        wl__w, f__w, ef__w, ssp_models_file,
        config_file=config_file,
        ssp_nl_fit_file=nl_ssp_models_file, sigma_inst=instrumental_dispersion, out_file=out_file,
        mask_list=mask_list, elines_mask_file=elines_mask_file,
        min=min, max=max, w_min=w_min, w_max=w_max, nl_w_min=nl_w_min, nl_w_max=nl_w_max,
        input_redshift=input_redshift, delta_redshift=delta_redshift,
        min_redshift=min_redshift, max_redshift=max_redshift,
        input_sigma=input_sigma, delta_sigma=delta_sigma, min_sigma=min_sigma, max_sigma=max_sigma,
        input_AV=input_AV, delta_AV=delta_AV, min_AV=min_AV, max_AV=max_AV,
        plot=plot, single_ssp=single_ssp, ratio=ratio, fit_sigma_rnd=fit_sigma_rnd,
        fit_gas=fit_gas, losvd_rnd_medres_merit=losvd_rnd_medres_merit,
        verbose=verbose, R_V=R_V, extlaw=extlaw,

        # TODO: need to be implemented in StPopSynth
        # sigma_in_AA=losvd_in_AA,

    )

    # write outputs
    if fit_gas:
        SPS.output_gas_emission(filename=out_file_elines)
    if not single_ssp:
        SPS.output_fits(filename=out_file_fit)
        SPS.output_coeffs_MC(filename=out_file_coeffs)
        SPS.output(filename=out_file, block_plot=False)
    else:
        SPS.output_single_ssp(filename=out_file_coeffs.replace('coeffs', 'chi_sq'))

    time_end = print_time(print_seed=False)
    time_total = time_end - time_ini_run
    print(f'# SECONDS = {time_total}')

def auto_ssp_elines_single(
    spec_file, ssp_file, out_file, config_file, plot=None,
    error_file=None, variance_error_column=False,
    ssp_nl_fit_file=None, sigma_inst=None, mask_list=None,
    min=None, max=None, w_min=None, w_max=None,
    nl_w_min=None, nl_w_max=None, elines_mask_file=None,
    input_redshift=None, delta_redshift=None, min_redshift=None, max_redshift=None,
    input_sigma=None, delta_sigma=None, min_sigma=None, max_sigma=None,
    input_AV=None, delta_AV=None, min_AV=None, max_AV=None, seed=None, ratio=None,
    fit_sigma_rnd=True, fit_gas=True):
    auto_ssp_spec(
        spec_file, ssp_file, out_file, config_file,
        error_file=error_file, nl_ssp_models_file=ssp_nl_fit_file,
        instrumental_dispersion=sigma_inst, mask_list=mask_list,
        elines_mask_file=elines_mask_file, plot=plot, min=min, max=max,
        wl_range=[w_min, w_max], nl_wl_range=[nl_w_min, nl_w_max],
        redshift_set=[input_redshift, delta_redshift, min_redshift, max_redshift],
        losvd_set=[input_sigma, delta_sigma, min_sigma, max_sigma],
        AV_set=[input_AV, delta_AV, min_AV, max_AV],
        seed=seed, ratio=ratio, fit_sigma_rnd=fit_sigma_rnd,
        variance_error_column=variance_error_column, fit_gas=fit_gas,

        # single_ssp
        single_ssp=True,

        # TODO: Needs to be implemented in StPopSynt
        losvd_in_AA=False,
    )
    #
    # ratio = True if ratio is None else ratio
    # ssp_nl_fit_file = ssp_file if ssp_nl_fit_file is None else ssp_nl_fit_file
    # out_file_elines = 'elines_' + out_file
    # out_file_single = 'single_' + out_file
    # out_file_coeffs = 'coeffs_' + out_file
    # out_file_fit = 'output.' + out_file + '.fits'
    # out_file_ps = out_file
    # time_ini_run = print_time(print_seed=False, get_time_only=True)
    # # initial time in seconds since Epoch.
    # seed = print_time() if seed is None else print_time(time_ini=seed)
    # # initial time used as the seed of the random number generator.
    # # np.random.seed(1573492732)
    # np.random.seed(seed)
    # # read arguments
    # clean_preview_results_files(out_file, out_file_elines, out_file_single, out_file_coeffs, out_file_fit)
    # wl__w, f__w, ef__w = read_spectra(spec_file, f_error=lambda x: 0.1*np.sqrt(np.abs(x)), variance_column=variance_error_column)
    # cf, SPS = auto_ssp_elines_single_main(
    #     wl__w, f__w, ef__w, ssp_file,
    #     config_file=config_file,
    #     ssp_nl_fit_file=ssp_nl_fit_file, sigma_inst=sigma_inst, out_file=out_file,
    #     mask_list=mask_list, elines_mask_file=elines_mask_file,
    #     min=min, max=max, w_min=w_min, w_max=w_max, nl_w_min=nl_w_min, nl_w_max=nl_w_max,
    #     input_redshift=input_redshift, delta_redshift=delta_redshift,
    #     min_redshift=min_redshift, max_redshift=max_redshift,
    #     input_sigma=input_sigma, delta_sigma=delta_sigma, min_sigma=min_sigma, max_sigma=max_sigma,
    #     input_AV=input_AV, delta_AV=delta_AV, min_AV=min_AV, max_AV=max_AV,
    #     plot=plot, single_ssp=True, ratio=ratio
    # )
    # # write outputs
    # SPS.output_gas_emission(filename=out_file_elines)
    # SPS.output_single_ssp(filename=out_file_coeffs.replace('coeffs', 'chi_sq'))
    # time_end = print_time(print_seed=False)
    # time_total = time_end - time_ini_run
    # print(f'# SECONDS = {time_total}')

# deprecated
def auto_ssp_elines_rnd(
    spec_file, ssp_file, out_file, config_file, plot=None,
    error_file=None, ssp_nl_fit_file=None, sigma_inst=None, mask_list=None,
    min=None, max=None, w_min=None, w_max=None,
    nl_w_min=None, nl_w_max=None, elines_mask_file=None,
    input_redshift=None, delta_redshift=None, min_redshift=None, max_redshift=None,
    input_sigma=None, delta_sigma=None, min_sigma=None, max_sigma=None,
    input_AV=None, delta_AV=None, min_AV=None, max_AV=None, seed=None, ratio=None,
    fit_sigma_rnd=True, variance_error_column=False, fit_gas=True):
    auto_ssp_spec(
        spec_file, ssp_file, out_file, config_file,
        error_file=error_file, nl_ssp_models_file=ssp_nl_fit_file,
        instrumental_dispersion=sigma_inst, mask_list=mask_list,
        elines_mask_file=elines_mask_file, plot=plot, min=min, max=max,
        wl_range=[w_min, w_max], nl_wl_range=[nl_w_min, nl_w_max],
        redshift_set=[input_redshift, delta_redshift, min_redshift, max_redshift],
        losvd_set=[input_sigma, delta_sigma, min_sigma, max_sigma],
        AV_set=[input_AV, delta_AV, min_AV, max_AV],
        seed=seed, ratio=ratio, fit_sigma_rnd=fit_sigma_rnd,
        variance_error_column=variance_error_column, fit_gas=fit_gas,

        # TODO: Needs to be implemented in StPopSynt
        losvd_in_AA=False,
    )

def load_rss(spec_file, error_file=None, output_seds=False):
    """Return the RSS from the given filename in the parsed command line arguments"""
    rss_f = fits.open(spec_file, memmap=False)
    if output_seds:
        rss_f_spectra = rss_f[0].data[0, :] - (rss_f[0].data[2, :] - rss_f[0].data[1, :])
    else:
        rss_f_spectra = rss_f[0].data
    if error_file is not None:
        rss_e = fits.open(error_file, memmap=False)
        rss_e_spectra = rss_e[0].data
        rss_e_spectra[~np.isfinite(rss_e_spectra)] = 1
    else:
        rss_e_spectra = 0.1*np.sqrt(np.abs(rss_f_spectra))
    wl__w = np.array([rss_f[0].header["CRVAL1"] + i*rss_f[0].header["CDELT1"] for i in range(rss_f[0].header["NAXIS1"])])
    return wl__w, rss_f_spectra, rss_e_spectra

def dump_rss_output(out_file_fit, wavelength, model_spectra):
    """Dump the RSS models into a FITS"""
    fits_name = out_file_fit if out_file_fit.endswith(".gz") else out_file_fit+".gz"
    # hdr = fits.Header()
    h = {}
    h['CRPIX1'] = 1
    h['CRVAL1'] = wavelength[0]
    h['CDELT1'] = wavelength[1] - wavelength[0]
    h['NAME0'] = 'org_spec'
    h['NAME1'] = 'model_spec'
    h['NAME2'] = 'mod_joint_spec'
    h['NAME3'] = 'gas_spec'
    h['NAME4'] = 'res_joint_spec'
    h['NAME5'] = 'no_gas_spec'
    h["FILENAME"] = fits_name
    h['COMMENT'] = f'OUTPUT {basename(sys.argv[0])} FITS'
    array_to_fits(fits_name, model_spectra, overwrite=True)
    write_img_header(fits_name, list(h.keys()), list(h.values()))
    # fits.PrimaryHDU(data=np.array(model_spectra).transpose(1,0,2), header=hdr).writeto(fits_name, overwrite=True)
    return model_spectra

def auto_ssp_elines_rnd_rss_main(
    wavelength, rss_flux, rss_eflux, output_files,
    ssp_file, config_file, out_file,
    ssp_nl_fit_file=None, sigma_inst=None, mask_list=None, elines_mask_file=None,
    min=None, max=None, w_min=None, w_max=None, nl_w_min=None, nl_w_max=None,
    input_redshift=None, delta_redshift=None, min_redshift=None, max_redshift=None,
    input_sigma=None, delta_sigma=None, min_sigma=None, max_sigma=None,
    input_AV=None, delta_AV=None, min_AV=None, max_AV=None,
    R_V=None, extlaw=None, plot=None, is_guided_sigma=False, ratio=True, spec_id=None,
    input_guided=None, input_guided_errors=None, fit_sigma_rnd=True, sps_class=StPopSynt):
    """Returns the model spectra, results and coefficient the analysis of a RSS.
    Also, returns the maps of the emission lines analysis."""
    elines_out, coeffs_out, summary_out = output_files
    guided_nl = False
    guided_errors = None
    if input_guided is not None:
        guided_nl = True
    # guided_sigma, sigma_seq = None, []
    sigma_seq = []
    input_delta_sigma = delta_sigma
    input_max_sigma = max_sigma
    input_min_sigma = min_sigma
    model_spectra = []
    results = []
    results_coeffs = []
    y_ratio = None
    ns = rss_flux.shape[0]
    output_el_models = {}
    if not guided_nl:
        _tmpcf = ConfigAutoSSP(config_file)
        for i_s in range(_tmpcf.n_systems):
            system = _tmpcf.systems[i_s]
            elcf = _tmpcf.systems_config[i_s]
            k = f'{system["start_w"]}_{system["end_w"]}'
            output_el_models[k] = create_emission_lines_parameters(elcf, ns)
        del _tmpcf
    for i, (f__w, ef__w) in enumerate(zip(rss_flux, rss_eflux)):
        print(f"\n# ID {i}/{ns - 1} ===============================================\n")
        if guided_nl:
            input_redshift = input_guided[0][i]
            delta_redshift = 0
            input_sigma = input_guided[1][i]
            delta_sigma = 0
            input_AV = input_guided[2][i]
            delta_AV = 0
            # (e_AV, e_sigma, e_redshift)
            guided_errors = (input_guided_errors[0][i], input_guided_errors[1][i], input_guided_errors[2][i])
            print(f'-> Forcing non-linear fit parameters (input guided):')
            print(f'-> input_guided_redshift:{input_redshift} e:{guided_errors[0]}')
            print(f'-> input_guided_sigma:{input_sigma} e:{guided_errors[1]}')
            print(f'-> input_guided_AV:{input_AV} e:{guided_errors[2]}')
        if i > 0 and (not guided_nl and is_guided_sigma):
            if SPS.best_sigma > 0:
                sigma_seq.append(SPS.best_sigma)
            guided_sigma = SPS.best_sigma
            k_seq = len(sigma_seq)
            n_seq_last = int(0.2*i)
            if n_seq_last < 10:
                n_seq_last = 10
            if k_seq > n_seq_last:
                guided_sigma = np.median(np.asarray(sigma_seq)[-n_seq_last:])
            input_sigma = guided_sigma
            min_sigma = guided_sigma - input_delta_sigma
            max_sigma = guided_sigma + input_delta_sigma
            delta_sigma = 0.25*input_delta_sigma
            if min_sigma < input_min_sigma:
                min_sigma = input_min_sigma
            if max_sigma > input_max_sigma:
                max_sigma = input_max_sigma
        cf, SPS = auto_ssp_elines_single_main(
            wavelength=wavelength, flux=f__w, eflux=ef__w, ssp_file=ssp_file, config_file=config_file,
            ssp_nl_fit_file=ssp_nl_fit_file, sigma_inst=sigma_inst, out_file=out_file,
            mask_list=mask_list, elines_mask_file=elines_mask_file,
            min=min, max=max, w_min=w_min, w_max=w_max, nl_w_min=nl_w_min, nl_w_max=nl_w_max,
            input_redshift=input_redshift, delta_redshift=delta_redshift,
            min_redshift=min_redshift, max_redshift=max_redshift,
            input_sigma=input_sigma, delta_sigma=delta_sigma, min_sigma=min_sigma, max_sigma=max_sigma,
            input_AV=input_AV, delta_AV=delta_AV, min_AV=min_AV, max_AV=max_AV,
            plot=plot, single_ssp=False,
            is_guided_sigma=is_guided_sigma,
            # guided_sigma=guided_sigma,
            spec_id=spec_id,
            guided_errors=guided_errors, ratio=ratio, y_ratio=y_ratio,
            fit_sigma_rnd=fit_sigma_rnd, sps_class=sps_class
        )
        y_ratio = SPS.ratio_master
        if not guided_nl:
            SPS.output_gas_emission(filename=elines_out, spec_id=i)
        SPS.output_coeffs_MC(filename=coeffs_out, write_header=i==0)
        SPS.output(filename=summary_out, write_header=i==0, block_plot=False)
        if not guided_nl:
            for system in SPS.config.systems:
                if system['EL'] is not None:
                    k = f'{system["start_w"]}_{system["end_w"]}'
                    append_emission_lines_parameters(system['EL'], output_el_models[k], i)
        model_spectra.append(SPS.output_spectra_list)
        results.append(SPS.output_results)
        results_coeffs.append(SPS.output_coeffs)

    # Transpose output
    # output model_spectra has dimensions (n_output_spectra_list, n_s, n_wavelength)
    model_spectra = np.array(model_spectra).transpose(1, 0, 2)
    # output results has dimensions (n_results, n_s)
    results = np.array(results).T
    # output results_coeffs has dimensions (n_results_coeffs, n_models, n_s)
    results_coeffs = np.array(results_coeffs).transpose(1, 2, 0)
    return model_spectra, results, results_coeffs, output_el_models

# TODO:
# def auto_ssp_rss: -> replaces auto_ssp_elines_rnd_rss
# def auto_ssp_cube

def auto_ssp_elines_rnd_rss(
    spec_file, ssp_file, out_file, config_file, plot=None,
    error_file=None, ssp_nl_fit_file=None, sigma_inst=None, mask_list=None,
    min=None, max=None, w_min=None, w_max=None,
    nl_w_min=None, nl_w_max=None, elines_mask_file=None,
    input_redshift=None, delta_redshift=None, min_redshift=None, max_redshift=None,
    input_sigma=None, delta_sigma=None, min_sigma=None, max_sigma=None,
    input_AV=None, delta_AV=None, min_AV=None, max_AV=None, seed=None, is_guided_sigma=False,
    ratio=None, guided_nl_file=None, sps_class=StPopSynt):
    ratio = True if ratio is None else ratio
    ssp_nl_fit_file = ssp_file if ssp_nl_fit_file is None else ssp_nl_fit_file
    out_file_elines = 'elines_' + out_file
    out_file_single = 'single_' + out_file
    out_file_coeffs = 'coeffs_' + out_file
    out_file_fit = 'output.' + out_file + '.fits.gz'
    out_file_ps = out_file
    time_ini_run = print_time(print_seed=False, get_time_only=True)
    # initial time in seconds since Epoch.
    seed = print_time() if seed is None else print_time(time_ini=seed)
    np.random.seed(seed)
    guided_nl = False
    input_guided = None
    input_guided_errors = None
    if guided_nl_file is not None:
        guided_nl = True
        AV, e_AV, redshift, e_redshift, sigma, e_sigma = np.genfromtxt(
            fname=guided_nl_file,
            comments="#", delimiter=",", usecols=(5,6,7,8,9,10), unpack=True
        )
        input_guided = (redshift, sigma, AV)
        input_guided_errors = (e_redshift, e_sigma, e_AV)
    # read arguments
    clean_preview_results_files(out_file, out_file_elines, out_file_single, out_file_coeffs, out_file_fit)
    # read RSS spectra
    wl__w, rss_f_spectra, rss_e_spectra = load_rss(spec_file, error_file, output_seds=guided_nl)
    # perform the RSS modelling
    with open(out_file_elines,"w") as elines_out, open(out_file_coeffs,"w") as coeffs_out, open(out_file,"w") as summary_out:
        model_spectra, _, _, _ = auto_ssp_elines_rnd_rss_main(
            wavelength=wl__w, rss_flux=rss_f_spectra, rss_eflux=rss_e_spectra,
            output_files=(elines_out,coeffs_out,summary_out),
            ssp_file=ssp_file, config_file=config_file,
            ssp_nl_fit_file=ssp_nl_fit_file, sigma_inst=sigma_inst, out_file=out_file,
            mask_list=mask_list, elines_mask_file=elines_mask_file,
            min=min, max=max, w_min=w_min, w_max=w_max, nl_w_min=nl_w_min, nl_w_max=nl_w_max,
            input_redshift=input_redshift, delta_redshift=delta_redshift,
            min_redshift=min_redshift, max_redshift=max_redshift,
            input_sigma=input_sigma, delta_sigma=delta_sigma, min_sigma=min_sigma, max_sigma=max_sigma,
            input_AV=input_AV, delta_AV=delta_AV, min_AV=min_AV, max_AV=max_AV,
            plot=plot, is_guided_sigma=is_guided_sigma, ratio=ratio,
            input_guided=input_guided, input_guided_errors=input_guided_errors, sps_class=sps_class
        )
    # write FITS RSS output
    dump_rss_output(out_file_fit=out_file_fit, wavelength=wl__w, model_spectra=model_spectra)
    time_end = print_time(print_seed=False)
    time_total = time_end - time_ini_run
    print(f'# SECONDS = {time_total}')



#!/usr/bin/env python3
import sys
from os import getcwd
from os.path import basename, splitext

from pyFIT3D.common.io import readFileArgumentParser
from pyFIT3D.common.io import create_ConfigAutoSSP_from_lists
from pyFIT3D.common.io import create_emission_lines_file_from_list
from pyFIT3D.common.gas_tools import detect_create_ConfigEmissionModel
from pyFIT3D.common.io import create_emission_lines_mask_file_from_list

def parse_arguments(default_args_file=None):
    """
    Parse the command line args.

    With fromfile_pidxrefix_chars=@ we can read and parse command line args
    inside a file with @file.txt.
    default args inside default_args_file
    """
    default_args = {
        'sigma_guess': 2.5,
        'nsearch': 1,
        'peak_threshold': 2,
        'dmin': 2,
        'plot': 0,
        'sort_by_flux': False,
        'output_path': getcwd(),
        'crossmatch_list_filename': None,
        'crossmatch_absdmax_AA': 6,
    }

    help_args = {
        'filename': 'Spectrum file.',
        'nconf': 'It will divide the spectrum in nconf chunks to generate nconf ConfigEmissionModel files.',
        'redshift': 'Used to correct the peaks detected to write the ConfigEmissionModel and also to compare the detected peaks with a crossmatch list when required.',
        'sigma_guess': 'Sigma guess to the emission line fit config.',
        'nsearch': 'A peak will have the highest flux around the interval (in pixels): (peak - nsearch, peak + nsearch)',
        'peak_threshold': 'Defines the standard deviation multiplicative factor for the probable peak threshold calculation,',
        'dmin': 'Defines the minimal distance between peaks (in pixels).',
        'plot': '0 = no plot | 1 = plot in screen | 2 = plot to file',
        'output_path': 'Output path to write config and masks files.',
        'sort_by_flux': 'Creates the ConfigEmissionModel file sorted by the flux peak (descending order).',
        'crossmatch_list_filename': 'A file with a list of known emission lines rest-frame central wavelengths to compare and match the detected peaks.',
        'crossmatch_absdmax_AA': 'Maximal absolute distance in Angstrom to identify a emission line when using a crossmatch list file.',
    }

    parser = readFileArgumentParser(fromfile_prefix_chars='@')

    parser.add_argument('--sigma_guess', '-S', metavar='FLOAT', type=float, default=default_args['sigma_guess'], help=help_args['sigma_guess'])
    parser.add_argument('--nsearch', '-N', metavar='INT', type=int, default=default_args['nsearch'], help=help_args['nsearch'])
    parser.add_argument('--peak_threshold', '-t', metavar='FLOAT', type=float, default=default_args['peak_threshold'], help=help_args['peak_threshold'])
    parser.add_argument('--dmin', '-d', metavar='INT', type=int, default=default_args['dmin'], help=help_args['dmin'])
    parser.add_argument('--crossmatch_absdmax_AA', metavar='INT', type=int, default=default_args['crossmatch_absdmax_AA'], help=help_args['crossmatch_absdmax_AA'])
    parser.add_argument('--output_path', '-O', metavar='OUTPUT_PATH', type=str, default=default_args['output_path'], help=help_args['output_path'])
    parser.add_argument('--crossmatch_list_filename', '-C', metavar='OUTPUT_PATH', type=str, default=default_args['crossmatch_list_filename'], help=help_args['crossmatch_list_filename'])
    parser.add_argument('--plot', '-p', type=int, choices=[0, 1, 2], help=help_args['plot'])
    parser.add_argument('--sort_by_flux', action='store_true', default=default_args['sort_by_flux'], help=help_args['sort_by_flux'])

    # positional args
    parser.add_argument('filename', metavar='SPECFILE', type=str, help=help_args['filename'])
    parser.add_argument('nconf', metavar='NCONF', type=int, help=help_args['nconf'])
    parser.add_argument('redshift', metavar='REDSHIFT', type=float, help=help_args['redshift'])

    parser.add_argument('--verbose', '-v', action='count', default=0)

    args_list = sys.argv[1:]
    # if exists file default.args, load default args
    if default_args_file is not None and isfile(default_args_file):
        args_list.insert(0, '@%s' % default_args_file)
    args = parser.parse_args(args=args_list)
    # TREAT ARGUMENTS HERE
    return args

def retrieve_spec(filename):
    from pyFIT3D.common.io import read_spectra
    # TODO READ CUBE AND FITS RSS
    _fname, _ext = splitext(filename)
    if _ext == '.fits' or ((_ext == '.gz') and (_fname[:-4] == 'fits')):
        print('Not implemented yed. TODO: read RSS and CUBE FITS')
    else:
        wl__w, f__w, _ = read_spectra(filename)
    return wl__w, f__w

def main(args):
    wave, flux = retrieve_spec(args.filename)
    r = detect_create_ConfigEmissionModel(wave, flux, redshift=args.redshift,
                                          sigma_guess=args.sigma_guess,
                                          chunks=args.nconf,
                                          peak_find_nsearch=args.nsearch,
                                          peak_find_threshold=args.peak_threshold,
                                          peak_find_dmin=args.dmin,
                                          flux_boundaries_fact=None,
                                          sigma_boundaries_fact=None,
                                          v0_boundaries_add=None,
                                          polynomial_order=None,
                                          polynomial_coeff_guess=None,
                                          polynomial_coeff_boundaries=None,
                                          plot=args.plot,
                                          output_path=args.output_path,
                                          sort_by_flux=args.sort_by_flux,
                                          crossmatch_list_filename=args.crossmatch_list_filename,
                                          crossmatch_absdmax_AA=args.crossmatch_absdmax_AA,
                                          verbose=args.verbose)
    config_filenames, wl_chunks, wave_peaks_tot, wave_peaks_tot_corr = r
    create_emission_lines_file_from_list(wave_peaks_tot, output_path=args.output_path)
    create_emission_lines_mask_file_from_list(wave_peaks_tot_corr, eline_half_range=8*args.sigma_guess, output_path=args.output_path)
    create_ConfigAutoSSP_from_lists(wl_chunks, config_filenames, output_path=args.output_path)

if __name__ == '__main__':
    # main(ReadArgumentsLocal())
    main(parse_arguments())





# list_eml = []
# list_eml_assoc = []
# list_eml_ind_assoc = []
# for w in wref:
#     absdist = np.abs(w - wl)
#     _where = np.where(absdist < 6)[0]
#     # while _where.size > 0:
#     tmp_wl = copy(wl)
#     tmp_ind_wl = np.arange(wl.size)
#     tmp_wl = tmp_wl.tolist()
#     tmp_ind_wl = tmp_ind_wl.tolist()
#     if _where.size > 0:
#         print(w, _where.size)
#         if _where.size == 1:
#             i_assoc = _where[0]
#             wl_assoc = wl[i_assoc]
#             print('w=', w, 'i_assoc=',i_assoc, 'wl_assoc=', wl_assoc)
#         else:
#             _iSwhere = np.argsort(absdist[_where])
#             #print(_where, absdist[_where], _iSwhere, absdist[_where[_iSwhere]])
#             #print(_where[_iSwhere[0]])
#             i_assoc = _where[_iSwhere[0]]
#             print('w=', w, 'i_assoc=',i_assoc, 'wl_assoc=', wl[i_assoc])
#         if w in list_eml:
#             i_list_eml = list_eml.index(w)
#             _prev_wl_assoc = wl[i_list_eml]
#             _prev_dist = np.abs(w - _prev_wl_assoc)
#             _dist = np.abs(w - wl_assoc)
#             print(_prev_dist, _dist)
#         else:
#             list_eml.append(w)
#             list_eml_assoc.append(wl_assoc)
#             list_eml_ind_assoc.append(i_assoc)



#!/usr/bin/env python3
################################################################################
#
# cont_seg_all_SN.py
# ----------------------
# Perl script: cont_seg_all_SN.pl
#
################################################################################

# System imports
import sys
from os.path import basename

# Local imports
from pyFIT3D.common.io import ReadArguments
from pyFIT3D.common.tools import cont_seg_all

class ReadArgumentsLocal(ReadArguments):
    """
    Argument parser for this script
    """
    __script_name__ = basename(sys.argv[0])
    __mandatory__ = ['signal_fits', 'flux_limit_peak', 'max_dist_peak', 'frac_peak', 'min_flux',
                     'segmentation_fits', 'diffuse_mask_fits']
    __arg_names__ = __mandatory__
    __N_tot_args__ = len(__arg_names__)
    __conv_func_mandatory__ = {'signal_fits': str, 'segmentation_fits': str, 'diffuse_mask_fits': str}
    __conv_func__ = __conv_func_mandatory__.copy()

    usage_msg_tmp1 = 'USE: {}'.format(__script_name__)
    usage_msg_tmp2 = ' Ha_map.fits FLUX_LIMIT_PEAK MAX_DIST_PEAK FRAC_PEAK MIN_FLUX SEGFILE.fits DIFFUSE_MASK.fits'
    __usage_msg__ = usage_msg_tmp1 + usage_msg_tmp2

    def __init__(self, args_list=None, verbose=False):
        ReadArguments.__init__(self, args_list, verbose=verbose)

if __name__ == '__main__':
    parsed_args = ReadArgumentsLocal()
    # sys.exit()
    cont_seg_all(parsed_args.signal_fits, parsed_args.flux_limit_peak,
                 parsed_args.max_dist_peak, parsed_args.frac_peak, parsed_args.min_flux,
                 parsed_args.segmentation_fits, parsed_args.diffuse_mask_fits)



#!/usr/bin/env python3
# System imports
import sys
import itertools
import numpy as np
from os.path import basename
from copy import deepcopy as copy

# Local imports
from pyFIT3D.common.io import ReadArguments
from pyFIT3D.common.io import array_to_fits, get_data_from_fits
from pyFIT3D.common.tools import clean_map

class ReadArgumentsLocal(ReadArguments):
    """
    Argument parser for this script
    """
    __script_name__ = basename(sys.argv[0])
    __mandatory__ = ['input1_fits', 'input2_fits',]
    __optional__ = ['max_vel', 'min_vel', 'output_fits']
    __arg_names__ = __mandatory__ + __optional__
    __N_tot_args__ = len(__arg_names__)
    __def_optional__ = {'max_vel': 1e12, 'min_vel': 0}
    __conv_func_mandatory__ = {'input1_fits': str, 'input2_fits': str}
    __conv_func_optional__ = {'output_fits': str}
    __conv_func__ = __conv_func_mandatory__.copy()
    __conv_func__.update(__conv_func_optional__)

    usage_msg_tmp1 = 'USE: {}'.format(__script_name__)
    usage_msg_tmp2 = ' INPUT_CUBE.FITS VELOCITY.FITS [MAX_VEL] [MIN_VEL] [OUTPUT_CUBE_NAME]'
    __usage_msg__ = usage_msg_tmp1 + usage_msg_tmp2

    def __init__(self, args_list=None, verbose=False):
        ReadArguments.__init__(self, args_list, verbose=verbose)
        # self._parse_filenames()

# from pyFIT3D.common.stats import _STATS_POS, pdl_stats
#
# def clean_cubes_map(in1_file, in2_file, max_vel, min_vel):
#     a_in1, h1 = get_data_from_fits(in1_file, header=True)
#     a_in2, h2 = get_data_from_fits(in2_file, header=True)
#     nz, ny, nx = a_in1.shape
#     mask_a_in2 = np.full((ny,nx), True, dtype=bool)
#     mask = (a_in2 > max_vel) | (a_in2<min_vel)
#     mask_a_in2[mask] = 0
#     for iyx in itertools.product(range(ny),range(nx)):
#         iy, ix = iyx
#         if (iy > 2) & (iy < (ny - 2)) & (ix > 2) & (ix < (nx - 2)):
#             if mask_a_in2[iy, ix] == 0:
#                 iy0 = iy - 1
#                 iy1 = iy + 2
#                 ix0 = ix - 1
#                 ix1 = ix + 2
#                 val = a_in1[:,iy0:iy1, ix0:ix1]
#                 val[~np.isfinite(val)] = 0
#                 val_vel = mask_a_in2[iy0:iy1, ix0:ix1]
#                 mask_vel = val_vel[val_vel>0]
#                 if mask_vel.sum() > 5:
#                     for iz in range(nz):
#                         val_z=a_in1[iz,iy0:iy1, ix0:ix1]
#                         val_z=val_z[val_vel>0]
#                         st_val_z = pdl_stats(val_z)
#                         a_in1[iz,iy, ix] = st_val_z[_STATS_POS['median']]
#     for iyx in itertools.product(range(ny),range(nx)):
#         iy, ix = iyx
#         if (iy > 2) & (iy < (ny - 2)) & (ix > 2) & (ix < (nx - 2)):
#             if a_in2[iy, ix] <= 0:
#                 iy0 = iy - 1
#                 iy1 = iy + 2
#                 ix0 = ix - 1
#                 ix1 = ix + 2
#                 val = a_in1[:,iy0:iy1, ix0:ix1]
#                 val[~np.isfinite(val)] = 0
#                 val_vel = mask_a_in2[iy0:iy1, ix0:ix1]
#                 mask_vel = val_vel[val_vel>0]
#                 if mask_vel.sum() > 5:
#                     for iz in range(nz):
#                         val_z=a_in1[iz,iy0:iy1, ix0:ix1]
#                         val_z=val_z[val_vel>0]
#                         st_val_z = pdl_stats(val_z)
#                         a_in1[iz,iy, ix] = st_val_z[_STATS_POS['median']]
#     array_to_fits(in1_file, a_in1, overwrite=True, header=h1)

def main(args):
    cube__zyx, h = get_data_from_fits(args.input1_fits, header=True)
    vel__yx = get_data_from_fits(args.input2_fits)
    nz = cube__zyx.shape[0]
    for iz in range(nz):
        print(f'cube map id: {iz}')
        map__yx = copy(cube__zyx[iz])
        cube__zyx[iz] = clean_map(map__yx, vel__yx,
                                  max_vel=args.max_vel, min_vel=args.min_vel)
    output_fits = args.input1_fits
    if args.output_fits is not None:
        output_fits = args.output_fits
    array_to_fits(output_fits, cube__zyx, overwrite=True, header=h)

if __name__ == '__main__':
    # pa = ReadArgumentsLocal()
    # clean_cubes_map(pa.input1_fits, pa.input2_fits, max_vel=pa.max_vel, min_vel=pa.min_vel)
    main(ReadArgumentsLocal(verbose=True))



#!/usr/bin/env python3
################################################################################
#
# get_slice.py
# ----------------------
# Perl script: get_slice.pl
#
################################################################################

# System imports
import sys
from os.path import basename

# Local imports
from pyFIT3D.common.io import ReadArguments
from pyFIT3D.common.io import array_to_fits
from pyFIT3D.common.io import get_data_from_fits
from pyFIT3D.common.io import get_wave_from_header
from pyFIT3D.common.tools import get_slice

class ReadArgumentsLocal(ReadArguments):
    """
    Local argument parser
    """
    __script_name__ = basename(sys.argv[0])
    __mandatory__ = ['input_cube_fits', 'prefix', 'conf_file']
    __optional__ = []
    __arg_names__ = __mandatory__ + __optional__
    __N_tot_args__ = len(__arg_names__)
    __conv_func_mandatory__ = {'input_cube_fits': str, 'prefix': str, 'conf_file': str}
    __conv_func_optional__ = {}
    __conv_func__ = __conv_func_mandatory__.copy()
    __conv_func__.update(__conv_func_optional__)

    __usage_msg__ = f'USE: {__script_name__}'
    __usage_msg__ += ' INPUT_CUBE.fits PREFIX CONF_FILE\n'
    __usage_msg__ += 'CONF_FILE: NAME START_W END_W'

    def __init__(self, args_list=None, verbose=False):
        ReadArguments.__init__(self, args_list, verbose=verbose)

def main(args):
    data__wyx, h = get_data_from_fits(args.input_cube_fits, header=True)
    wave__w = get_wave_from_header(h, wave_axis=3)
    slices = get_slice(data__wyx, wave__w, args.prefix, args.conf_file)
    if len(slices) > 0:
        for k, v in slices.items():
            if v is not None:
                fname = f'{k}.fits'
                array_to_fits(fname, v, overwrite=True)
                print(f'{fname} saved')
            else:
                s_w, e_w = k.split('_')[-2:]
                print(f'section ({s_w}, {e_w}) out of margins')

if __name__ == '__main__':
    main(ReadArgumentsLocal())
    # parsed_args.input_cube_fits, parsed_args.prefix, parsed_args.conf_file)



#!/usr/bin/env python3
import os
import sys
import gzip
import glob
import shutil
import itertools
import numpy as np
import configparser
from os import makedirs, remove
from copy import deepcopy as copy
from os.path import basename, isfile
from scipy.ndimage import median_filter
from contextlib import redirect_stdout, redirect_stderr

from pyFIT3D.common.tools import flux_elines_cube_EW
from pyFIT3D.common.auto_ssp_tools import ConfigAutoSSP
from pyFIT3D.common.tools import array_to_fits, get_wave_from_header, pack_results_name
from pyFIT3D.common.tools import trim_waves, get_data_from_fits, vel_eline, clean_Ha_map, imarith
from pyFIT3D.common.gas_tools import output_emission_lines_spectra, create_emission_lines_parameters
from pyFIT3D.common.io import ReadArguments, print_time, remove_isfile, print_done, print_block_init
from pyFIT3D.common.constants import _MODELS_ELINE_PAR, __c__, __FWHM_to_sigma__, __sigma_to_FWHM__, __Ha_central_wl__
from pyFIT3D.common.gas_tools import kin_cube_elines_main, output_emission_lines_parameters, create_emission_lines_parameters

class ReadArgumentsLocal(ReadArguments):
    """
    Argument parser for this script
    """
    __script_name__ = basename(sys.argv[0])

    __mandatory__ = ['name', 'config_file']
    __optional__ = ['dir_out', 'dir_datacubes_out']
    __arg_names__ = __mandatory__ + __optional__
    __N_tot_args__ = len(__arg_names__)

    __conv_func_mandatory__ = {'name': str, 'config_file': str}
    __conv_func_optional__ = {'dir_out': str, 'dir_datacubes_out': str}
    __conv_func__ = __conv_func_mandatory__.copy()
    __conv_func__.update(__conv_func_optional__)

    usage_msg_tmp1 = 'USE: {}'.format(__script_name__)
    usage_msg_tmp2 = ' NAME CONFIG_FILE [DIR_OUT] [DIR_DATACUBES_OUT]'
    usage_msg_tmp3 = '\nCONFIG_FILE is mandatory but defaults to ana_single_kin.ini'
    __usage_msg__ = usage_msg_tmp1 + usage_msg_tmp2 + usage_msg_tmp3

    def __init__(self, args_list=None, verbose=False):
        ReadArguments.__init__(self, args_list, verbose=verbose)
        self._parse()

    def _parse(self):
        if self.config_file is None:
            self.config_file = 'ana_single.ini'
        if not isfile(self.config_file):
            print(f'{self.__script_name__}: {self.config_file}: not found')
            sys.exit()

def ana_single_kin(name, input_config, input_dir_out=None, input_dir_datacubes_out=None):
    config = input_config['general']
    config_elines = input_config['emission_lines']
    config_pack = input_config['pack']
    config_seg = input_config['cube_segmentation']
    cube_filename = config['cube_filename'].replace('NAME', name)
    plot = config.getint('plot', 0)
    save_aux_files = config.getboolean('save_aux_files', True)
    dir_conf = config.get('dir_conf', '../legacy')
    dir_out = config.get('dir_out', '../out')
    # Try to set a new dir_datacubes_out
    dir_datacubes_out = input_dir_datacubes_out
    if dir_datacubes_out is None:
        dir_datacubes_out = config.get('dir_datacubes_out', '../out')
    inst_disp = config.getfloat('instrumental_dispersion')

    if not dir_out.endswith('/'):
        dir_out += '/'
    if not dir_datacubes_out.endswith('/'):
        dir_datacubes_out += '/'
    if 'manga' in name:
        _, plate, ifu = name.split('-')
        dir_out += f'{plate}/{ifu}/'
    else:
        dir_out += f'{name}/'

    ##########################################################################
    # Load datacube
    #
    time_ini_loc = print_block_init('Loading datacube and redshift information...')
    org_cube__wyx, org_h, n_ext = get_data_from_fits(cube_filename, header=True, return_n_extensions=True)
    ny, nx = org_cube__wyx.shape[1:]
    h_set_cube = {'CRVAL3': org_h['CRVAL3'], 'CDELT3': org_h['CDELT3'], 'CRPIX3': org_h['CRPIX3']}
    org_error__wyx = None
    if org_h['EXTEND']:
        org_error__wyx = get_data_from_fits(cube_filename, extension=1)
    else:
        print('# Unable to find an EXTEND key on cube: ', flush=True)
        print('# - missing error spectra', flush=True)

    # original wavelength
    wave_org__w = get_wave_from_header(org_h, wave_axis=3)
    auto_ssp_config_file = f'{dir_out}auto.{name}.config'
    gas_cube_filename = config_seg.get('gas_cube_file', 'GAS.NAME.cube.fits').replace('NAME', name)
    gas_cube_filename = f'{dir_out}{gas_cube_filename}.gz'
    print(f'# Gas cube = {gas_cube_filename}')
    gas_cube__wyx = get_data_from_fits(gas_cube_filename)

    # reading redshift from auto_ssp_Z
    tmp_cf = input_config['cen_auto_ssp_no_lines']
    auto_ssp_output_filename = tmp_cf.get('output_filename', 'auto_ssp_Z.NAME.cen.out').replace('NAME', name)
    auto_ssp_output_filename = f'{dir_out}{auto_ssp_output_filename}'
    red = np.loadtxt(fname=auto_ssp_output_filename, unpack=True, comments='#', delimiter=',', usecols=12)
    red_f = 300/__c__
    min_red = red - red_f
    max_red = red + red_f
    d_red = 0.1*red_f
    vel = red*__c__
    print(f'# Redshift = {red:.8f} - Vel = {vel:.8f}')
    print_done(time_ini=time_ini_loc)

    # Read SSP MOD
    ssp_mod_filename = config_seg.get('ssp_mod_file', 'SSP_mod.NAME.cube.fits').replace('NAME', name)
    ssp_mod_filename = f'{dir_out}{ssp_mod_filename}.gz'
    ssp_mod_cube__wyx = get_data_from_fits(ssp_mod_filename)

    # elines file
    tmp_cf = input_config['auto_ssp_rss_guided']
    elines_file = tmp_cf.get('output_filename', 'auto_ssp.CS.NAME.rss.out').replace('NAME', name)
    elines_file = f'{dir_out}elines_{elines_file}'
    print(f'# elines file = {elines_file}')

    # SN_mask_map
    SN_mask_map_filename = f'{dir_out}mask.{name}.V.fits'
    ##########################################################################
    # Emission Lines analysis
    #
    time_ini_loc = print_block_init('Emission Lines analysis...')
    wave_ref = config_elines.getfloat('vel_eline_reference_wavelength', __Ha_central_wl__)
    vel_hr = config_elines.getfloat('vel_eline_half_range', 200)
    nsearch = config_elines.getint('vel_eline_nsearch', 1)
    imin = config_elines.getfloat('vel_eline_imin', 0.95)
    w_min = wave_ref*(1 + min_red - vel_hr/__c__)
    w_max = wave_ref*(1 + max_red + vel_hr/__c__)
    print('# -----=> vel_eline_cube()', flush=True)
    vel_eline_map_filename = config_elines.get('vel_eline_map_file', 've.NAME.vel_map.fits').replace('NAME', name)
    vel_eline_mask_filename = config_elines.get('vel_eline_mask_map_file', 've.NAME.mask_map.fits').replace('NAME', name)
    vel_map__yx = np.zeros((ny, nx))
    vel_mask_map__yx = np.zeros((ny, nx))
    sel_wl = trim_waves(wave_org__w, [w_min, w_max])
    wave = wave_org__w[sel_wl]
    for ixy in itertools.product(range(nx),range(ny)):
        ix, iy = ixy
        flux = gas_cube__wyx[sel_wl, iy, ix]
        ve, mask, npeaks = vel_eline(flux, wave, nsearch=nsearch, imin=imin,
                                      wave_ref=wave_ref, set_first_peak=False)
        vel_map__yx[iy, ix] = ve
        vel_mask_map__yx[iy, ix] = mask
    vel_hr = config_elines.getfloat('clean_Ha_map_vel_half_range', 500)
    min_vel = red*__c__ - vel_hr
    max_vel = red*__c__ + vel_hr
    print('# -----=> clean_Ha_map()', flush=True)
    vel_map_clean__yx, vel_mask_map_clean__yx = clean_Ha_map(copy(vel_map__yx), copy(vel_mask_map__yx),
                                                             min_vel=min_vel, max_vel=max_vel)
    h_set_vel = {
        'COMMENT': 'vel_eline_cube result',
        'WMIN': w_min,
        'WMAX': w_max,
        'WREF': wave_ref,
        'FILENAME': vel_eline_map_filename,
    }
    array_to_fits(vel_eline_map_filename, vel_map_clean__yx, overwrite=True, header=h_set_vel)
    h_set_vel['FILENAME'] = vel_eline_mask_filename
    array_to_fits(vel_eline_mask_filename, vel_mask_map_clean__yx, overwrite=True, header=h_set_vel)

    # auto_ssp_elines_rnd_rss_main output() by file.
    with open(elines_file, 'r') as f:
        line = f.readline()
        while not line.startswith('eline'):
            line = f.readline()
        disp_max_elines = eval(line.split(' ')[5])*1.5
    print(f'disp_max_elines = {disp_max_elines}')
    dispersion_guess = config_elines.getfloat('dispersion_guess', 1.2)
    dispersion_min = config_elines.getfloat('dispersion_min', 0.5)
    input_config_file = config_elines.get('auto_ssp_config_file')
    # Preparing config to kin_cube_elines
    cf = ConfigAutoSSP(input_config_file)
    vel_hr = config_elines.getfloat('vel_half_range', 300)
    vel_min = vel - vel_hr
    vel_max = vel + vel_hr
    vel_map = copy(vel_map_clean__yx)
    vel_mask_map = copy(vel_mask_map_clean__yx)
    sigma_map = None
    sigma_fixed = None
    for i_s in range(cf.n_systems):
        syst = cf.systems[i_s]
        z_fact = 1 + red
        w_min = syst['start_w']
        w_max = syst['end_w']
        w_min_corr = int(w_min*z_fact)
        w_max_corr = int(w_max*z_fact)
        elcf = cf.systems_config[i_s]
        # v0
        elcf.guess[0][_MODELS_ELINE_PAR['v0']] = vel
        elcf.to_fit[0][_MODELS_ELINE_PAR['v0']] = 1
        elcf.pars_0[0][_MODELS_ELINE_PAR['v0']] = vel_min
        elcf.pars_1[0][_MODELS_ELINE_PAR['v0']] = vel_max
        elcf.links[0][_MODELS_ELINE_PAR['v0']] = -1
        # sigma
        elcf.guess[0][_MODELS_ELINE_PAR['sigma']] = dispersion_guess
        elcf.to_fit[0][_MODELS_ELINE_PAR['sigma']] = 1
        elcf.pars_0[0][_MODELS_ELINE_PAR['sigma']] = dispersion_min
        elcf.pars_1[0][_MODELS_ELINE_PAR['sigma']] = disp_max_elines
        elcf.links[0][_MODELS_ELINE_PAR['sigma']] = -1
        if save_aux_files:
            # print config to file
            fname = f'tmp.{name}.{i_s}.config'
            elcf.print(filename=fname)
        w_min = int(w_min)
        w_max = int(w_max)
        w_minmax_prefix = f'{w_min}_{w_max}'
        print(f'w_minmax_prefix = {w_minmax_prefix}')
        _m_str = 'model'
        if elcf.n_models > 1:
            _m_str += 's'
        print(f'# analyzing {elcf.n_models} {_m_str} in {w_minmax_prefix.replace("_", "-")} wavelength range')
        output_filename = config_elines.get('output_file', f'KIN.GAS.prefix.NAME.out').replace('NAME', name)
        output_filename = output_filename.replace('prefix', w_minmax_prefix)
        f_log = None
        output_log_filename = config_elines.get('log_file', None)
        if output_log_filename is not None:
            output_log_filename = output_log_filename.replace('NAME', name)
            output_log_filename = output_log_filename.replace('prefix', w_minmax_prefix)
            f_log = open(output_log_filename, 'w')
        map_output_prefix = config_elines.get('map_output_prefix', f'map.prefix.NAME').replace('NAME', name).replace('prefix', w_minmax_prefix)
        map_output_prefix = map_output_prefix.replace('prefix', w_minmax_prefix)
        fix_disp = 0
        if w_min == 3700:
            sigma_fixed = 1
        print('# -----=> kin_cube_elines_rnd()', flush=True)
        time_ini_kin = print_time(print_seed=False)
        sel_wl_range = trim_waves(wave_org__w, [w_min_corr, w_max_corr])
        if sel_wl_range.astype('int').sum() == 0:
            print('[kin_cube_elines]: n_wavelength = 0: No avaible spectra to perform the configured analysis.')
            output_el_models = create_emission_lines_parameters(elcf, shape=(ny, nx))
        else:
            wave_msk__w = copy(wave_org__w[sel_wl_range])
            flux_msk__wyx = copy(gas_cube__wyx[sel_wl_range])
            eflux_msk__wyx = None
            if org_error__wyx is not None:
                eflux_msk__wyx = copy(org_error__wyx[sel_wl_range])
            n_MC = config_elines.getint('n_MonteCarlo', 30)
            n_loops = config_elines.getint('n_loops', 3)
            scale_ini = config_elines.getfloat('scale_ini', 0.15)
            with redirect_stdout(f_log):
                r = kin_cube_elines_main(
                    wave_msk__w, flux_msk__wyx, elcf, output_filename,
                    cube_eflux=eflux_msk__wyx, n_MC=n_MC, n_loops=n_loops, scale_ini=scale_ini,
                    vel_map=vel_map, sigma_map=sigma_map,
                    vel_fixed=0, sigma_fixed=sigma_fixed,
                    redefine_max=True, vel_mask_map=vel_mask_map, memo=0,
                    plot=plot,
                    run_mode=config_elines.get('run_mode', 'RND'),
                )
            if f_log is not None:
                f_log.close()
            output_el_spectra, output_el_models = r
            output_emission_lines_spectra(wave_msk__w, output_el_spectra, h_set_cube, map_output_prefix.replace('map', 'KIN.cube'))
        output_emission_lines_parameters(map_output_prefix, elcf, output_el_models)
        print_done(time_ini=time_ini_kin, message='DONE kin_cube_elines_rnd()!')

        if not i_s:
            NIIHa_output_el_models = output_el_models
            print('# -----=> med2df() v_Ha map', flush=True)
            index_model_Ha = 0
            vel_map_Ha__yx = NIIHa_output_el_models['v0'][index_model_Ha]
            vel_map_Ha__yx[~np.isfinite(vel_map_Ha__yx)] = 0
            vel_map = copy(vel_map_Ha__yx)
            vel_map_Ha__yx = median_filter(copy(vel_map_Ha__yx), size=(3, 3), mode='reflect')
            # med2df(f'{map_output_prefix}_vel_00.fits', f'vel.{name}.fits', 3, 3)
            sigma_map_Ha__yx = NIIHa_output_el_models['sigma'][index_model_Ha]
            print('# -----=> med2df() disp_Ha map', flush=True)
            sigma_map_Ha__yx[~np.isfinite(sigma_map_Ha__yx)] = 0
            sigma_map = copy(sigma_map_Ha__yx)
            sigma_map_flux_elines__yx = copy(sigma_map_Ha__yx)
            disp_map_Ha_filtered__yx = median_filter(sigma_map_Ha__yx*__sigma_to_FWHM__, size=(3, 3), mode='reflect')
            # med2df(f'{map_output_prefix}_disp_00.fits', f'disp.{name}.fits', 3, 3)
            print('# -----=> imarith()', flush=True)
            # imarith(f'disp.{name}.fits', '*', str(__FWHM_to_sigma__), f'disp.{name}.fits')
            if save_aux_files:
                vel_filename = f'vel.{name}.fits'
                remove_isfile(vel_filename)
                array_to_fits(vel_filename, vel_map_Ha__yx, overwrite=True)
                disp_filename = f'disp.{name}.fits'
                remove_isfile(disp_filename)
                sigma_map_Ha_filtered__yx = __FWHM_to_sigma__*disp_map_Ha_filtered__yx
                array_to_fits(disp_filename, sigma_map_Ha_filtered__yx, overwrite=True)
        print(f'# emission lines in wavelength range {w_minmax_prefix.replace("_", "-")} analyzed')
    print_done(time_ini=time_ini_loc)
    ##########################################################################

    ##########################################################################
    # gzip fits
    #
    time_ini_loc = print_block_init('Compressing FITS files...')
    compresslevel = config.getint('gzip_compresslevel', 6)
    for file in glob.glob(f'*{name}*.fits'):
        with open(file, 'rb') as f_in:
            # EL: compresslevel 6 is biased towars high compression
            # at expense of speed, also 6 is also the value of gzip
            # in shell mode.
            with gzip.open(f'{file}.gz', mode='wb', compresslevel=compresslevel) as f_comp:
                shutil.copyfileobj(f_in, f_comp)
        # remove uncompressed file
        remove(file)
    print_done(time_ini=time_ini_loc)
    ##########################################################################

    ##########################################################################
    # Momentum analysis of emission lines
    #
    time_ini_loc = print_block_init('Momentum analysis of emission lines (flux_elines)...')
    tmp_cf = input_config['flux_elines']
    elines_list = tmp_cf.get('elines_list')
    flux_elines_output_filename = tmp_cf.get('output_file', 'flux_elines.NAME.cube.fits.gz').replace('NAME', name)
    print('# -----=> flux_elines_cube_EW()', flush=True)

    fe_output, fe_header = flux_elines_cube_EW(
        flux__wyx = gas_cube__wyx,
        input_header = h_set_cube,
        n_MC=tmp_cf.getint('n_MonteCarlo', 10),
        elines_list=elines_list,
        vel__yx=vel_map_clean__yx,
        sigma__yx=sigma_map_flux_elines__yx,
        eflux__wyx=org_error__wyx,
        flux_ssp__wyx=ssp_mod_cube__wyx,
    )
    array_to_fits(flux_elines_output_filename, fe_output, header=fe_header, overwrite=True)
    print_done(time_ini=time_ini_loc)
    ##########################################################################


    ##########################################################################
    # Final pack, and store files
    #
    time_ini_final_pack = print_block_init('Begin final pack of files...')

    ##########################################################################
    # Mask maps
    #
    time_ini_loc = print_block_init('Masking maps...')
    for fname in glob.glob(f'map.CS*{name}*.fits.gz'):
        print(f' ---=> imarith() {fname}', flush=True)
        imarith(fname, '*', f'{SN_mask_map_filename}.gz', fname)
    print_done(time_ini=time_ini_loc)
    ##########################################################################

    ##########################################################################
    # Pack files
    #
    time_ini_loc = print_block_init('Packing files...')
    print('# -----=> pack_results_name()', flush=True)
    pack_results_name(name,
                      pack_filename=config_pack.get('ELINES_file'),
                      prefix='ELINES')
    print_done(time_ini=time_ini_loc)
    ##########################################################################

    ##########################################################################
    # copy and move files
    # the filename should be in dst in order to rewrite file

    if input_dir_out is not None:
        dir_out = input_dir_out
        if not dir_out.endswith('/'):
            dir_out += '/'
        if 'manga' in name:
            try:
                _, plate, ifu = name.split('-')
                dir_out += f'{plate}/{ifu}/'
            except:
                dir_out += f'{name}/'
        else:
            dir_out += f'{name}/'
        try:
            makedirs(dir_out)
        except FileExistsError:
            print(f'{basename(sys.argv[0])}: {dir_out}: directory already exists')
    if not dir_datacubes_out.endswith('/'):
        dir_datacubes_out += '/'
    try:
        makedirs(dir_datacubes_out)
    except FileExistsError:
        print(f'{basename(sys.argv[0])}: {dir_datacubes_out}: directory already exists')
    print(f'Moving files to {dir_out}', flush=True)
    for file in glob.glob(f'*{name}*'):
        shutil.move(src=file, dst=f'{dir_out}{file}')
    print_done(time_ini=time_ini_loc, message='DONE move files!')
    print(f'Copying datacube to {dir_datacubes_out}', flush=True)
    files_to_copy = [
        f'{name}.ELINES.cube.fits.gz',
        flux_elines_output_filename,
    ]
    for file in files_to_copy:
        shutil.copy(src=f'{dir_out}{file}', dst=f'{dir_datacubes_out}')
    print_done(time_ini=time_ini_loc, message='DONE: store files!')
    ##########################################################################

    print_done(time_ini=time_ini_final_pack, message='DONE: final pack!')
    ##########################################################################

    return dir_out

def main(args):
    name = args.name
    config = configparser.ConfigParser(
        # allow a variables be set without value
        allow_no_value=True,
        # allows duplicated keys in different sections
        strict=False,
        # deals with variables inside configuratio file
        interpolation=configparser.ExtendedInterpolation())
    config.read(args.config_file)
    dir_log = 'log'
    log_file = config['general'].get('kin_log_file', None)
    f = None
    if (log_file is not None):
        if 'NAME' in log_file:
            log_file = log_file.replace('NAME', name)
            omode = 'w'
        else:
            omode='a'
        if log_file[0:5] != '/dev/':
            try:
                makedirs(dir_log)
                log_file = f'{dir_log}/{log_file}'
            except FileExistsError:
                print(f'{basename(sys.argv[0])}: {dir_log}: directory already exists')
                log_file = f'{dir_log}/{log_file}'
            except:
                print(f'{basename(sys.argv[0])}: {dir_log}: cannot create directory')
        else:
            omode = 'w'
        f = open(log_file, omode)
    with redirect_stdout(f):
        with redirect_stderr(f):
            init_message = f'Initiating run for galaxy {name} - May the force be with you!'
            init_message += f'\n# Configuration file = {args.config_file}'
            init_message += f'\n# Output log file = {log_file}'
            cube_filename = config['general']['cube_filename'].replace('NAME', name)
            init_message += f'\n# Cube file = {cube_filename}'
            time_ini_run = print_block_init(init_message, print_seed=True)
            seed = config['general'].getint('seed', None)
            if seed is not None:
                print(f'# Forcing seed: {seed}')
            else:
                seed = time_ini_run
            np.random.seed(seed)
            dir_out = ana_single_kin(name, config,
                                     input_dir_out=args.dir_out,
                                     input_dir_datacubes_out=args.dir_datacubes_out)
            print_done(time_ini=time_ini_run, message=f'DONE: galaxy {name} analyzed.')
    # After some problems with the write in the log_file
    # we move the log_file after everithing is done.
    # If the program was not able to create the dir_log directory
    # in some machines the final writting of the log file
    # could be incomplete.
    if (log_file is not None) and os.path.isfile(log_file):
        f.close()
        if name in log_file:
            _tmp = config['general'].get('kin_log_file', 'ana_single_kin.NAME.log').replace('NAME', name)
            shutil.move(src=log_file, dst=f'{dir_out}{_tmp}')

if __name__ == '__main__':
    main(ReadArgumentsLocal())



#!/usr/bin/env python3
################################################################################
#
# csv_to_map.seg.py
# ----------------------
# Perl script: csv_to_map.seg.pl
#
################################################################################

# System imports
import sys
from os.path import basename

# Local imports
from pyFIT3D.common.io import ReadArguments
from pyFIT3D.common.tools import csv_to_map_seg

class ReadArgumentsLocal(ReadArguments):
    """
    Argument parser for this script
    """
    __script_name__ = basename(sys.argv[0])
    __mandatory__ = ['csvfile', 'column', 'seg_file_fits', 'map_fits']
    __arg_names__ = __mandatory__
    __N_tot_args__ = len(__arg_names__)
    __conv_func_mandatory__ = {'csvfile': str, 'seg_file_fits': str,
                               'map_fits': str}
    __conv_func__ = __conv_func_mandatory__.copy()

    usage_msg_tmp1 = 'USE: {}'.format(__script_name__)
    usage_msg_tmp2 = ' CSVFILE column seg_file.fits map.fits'
    __usage_msg__ = usage_msg_tmp1 + usage_msg_tmp2

    def __init__(self, args_list=None, verbose=False):
        ReadArguments.__init__(self, args_list, verbose=verbose)
        # self._parse_filenames()

if __name__ == '__main__':
    parsed_args = ReadArgumentsLocal()
    csvfile = parsed_args.csvfile
    column = int(parsed_args.column)
    seg_file_fits = parsed_args.seg_file_fits
    map_fits = parsed_args.map_fits
    csv_to_map_seg(csvfile, column, seg_file_fits, map_fits)



#!/usr/bin/env python3
################################################################################
#
# get_SN_cube.py
# ----------------------
# Perl script: get_SN_cube.pl
#
################################################################################

# System imports
import sys
from os.path import basename

# Local imports
from pyFIT3D.common.io import ReadArguments
from pyFIT3D.common.io import array_to_fits
from pyFIT3D.common.io import get_data_from_fits
from pyFIT3D.common.io import get_wave_from_header
from pyFIT3D.common.tools import get_SN_cube

class ReadArgumentsLocal(ReadArguments):
    """
    Local argument parser
    """
    __script_name__ = basename(sys.argv[0])
    __mandatory__ = ['input_cube_fits', 'output_SN_map', 'output_signal_map', 'output_noise_map']
    __optional__ = ['w_minmax']
    __def_optional__ = {'w_minmax': '0,0'}
    __arg_names__ = __mandatory__ + __optional__
    __N_tot_args__ = len(__arg_names__)
    __conv_func_mandatory__ = {'input_cube_fits': str, 'output_SN_map': str,
                               'output_signal_map': str, 'output_noise_map': str}
    __conv_func_optional__ = {'w_minmax': str}
    __conv_func__ = __conv_func_mandatory__.copy()
    __conv_func__.update(__conv_func_optional__)

    __usage_msg__ = f'USE: {__script_name__}'
    __usage_msg__ += ' input_cube.fits SN_map.fits signalmap.fits noisemap.fits [WMIN,WMAX]'

    def __init__(self, args_list=None, verbose=False):
        ReadArguments.__init__(self, args_list, verbose=verbose)
        if self.w_minmax is not None:
            self.parse_wavelength()

    def parse_wavelength(self):
        wminmax = self.w_minmax.split(',')
        if len(wminmax) == 2:
            self.wmin = eval(wminmax[0])
            self.wmax = eval(wminmax[1])

def main(args):
    # read data
    org_cube__wyx, h = get_data_from_fits(args.input_cube_fits, header=True)

    # create wavelength array
    wave__w = get_wave_from_header(h, wave_axis=3)

    SN__yx, mean_flux__yx, sigma_flux__yx = get_SN_cube(org_cube__wyx, wave__w, args.wmin, args.wmax)

    array_to_fits(filename=args.output_SN_map, arr=SN__yx, overwrite=True)
    array_to_fits(filename=args.output_signal_map, arr=mean_flux__yx, overwrite=True)
    array_to_fits(filename=args.output_noise_map, arr=sigma_flux__yx, overwrite=True)

if __name__ == '__main__':
    main(ReadArgumentsLocal())



#!/usr/bin/env python3
################################################################################
#
# map_auto_ssp_AGE_MET_rnd.seg.py
# ----------------------
# Perl script: map_auto_ssp_AGE_MET_rnd.seg.pl
#
################################################################################

# System imports
import sys
from os.path import basename

# Local imports
from pyFIT3D.common.io import ReadArguments
from pyFIT3D.common.tools import map_auto_ssp_AGE_MET_rnd_seg

class ReadArgumentsLocal(ReadArguments):
    """
    Local argument parser
    """
    __script_name__ = basename(sys.argv[0])
    __mandatory__ = ['input', 'segmentation_fits', 'output_prefix', 'central_wavelength', 'inst_disp']
    # __optional__ = ['output_diffuse_fits']
    # __def_optional__ = {'output_diffuse_fits': 'diffuse.fits'}
    __arg_names__ = __mandatory__  # + __optional__
    __N_tot_args__ = len(__arg_names__)
    __conv_func_mandatory__ = {'input': str, 'segmentation_fits': str, 'output_prefix': str}
    # __conv_func_optional__ = {'output_diffuse_fits': str}
    __conv_func__ = __conv_func_mandatory__.copy()
    # __conv_func__.update(__conv_func_optional__)

    __usage_msg__ = f'USE: {__script_name__}'
    __usage_msg__ += ' auto_ssp.out seg_file.fits PREFIX_OUT WAVE_CEN INST_DISP'

    def __init__(self, args_list=None, verbose=False):
        ReadArguments.__init__(self, args_list, verbose=verbose)

if __name__ == '__main__':
    pa = ReadArgumentsLocal()
    map_auto_ssp_AGE_MET_rnd_seg(pa.input, pa.segmentation_fits, pa.output_prefix,
                                 pa.central_wavelength, pa.inst_disp)
